@@ -42,31 +42,26 @@
 """
 Handles basic connections to AWS
 """
-
-from __future__ import with_statement
-import base64
+from datetime import datetime
 import errno
-import httplib
 import os
-import Queue
 import random
 import re
 import socket
 import sys
 import time
-import urllib
-import urlparse
 import xml.sax
 import copy
 
-import auth
-import auth_handler
+from boto import auth
+from boto import auth_handler
 import boto
 import boto.utils
 import boto.handler
 import boto.cacerts
 
 from boto import config, UserAgent
+from boto.compat import six, http_client, urlparse, quote, encodebytes
 from boto.exception import AWSConnectionError
 from boto.exception import BotoClientError
 from boto.exception import BotoServerError
@@ -95,7 +90,7 @@ ON_APP_ENGINE = all(key in os.environ for key in (
 PORTS_BY_SECURITY = {True: 443,
                      False: 80}
 
-DEFAULT_CA_CERTS_FILE = os.path.join(os.path.dirname(os.path.abspath(boto.cacerts.__file__ )), "cacerts.txt")
+DEFAULT_CA_CERTS_FILE = os.path.join(os.path.dirname(os.path.abspath(boto.cacerts.__file__)), "cacerts.txt")
 
 
 class HostConnectionPool(object):
@@ -164,7 +159,7 @@ class HostConnectionPool(object):
 
     def _conn_ready(self, conn):
         """
-        There is a nice state diagram at the top of httplib.py.  It
+        There is a nice state diagram at the top of http_client.py.  It
         indicates that once the response headers have been read (which
         _mexe does before adding the connection to the pool), a
         response is attached to the connection, and it stays there
@@ -369,34 +364,37 @@ class HTTPRequest(object):
                  self.headers, self.body))
 
     def authorize(self, connection, **kwargs):
-        for key in self.headers:
-            val = self.headers[key]
-            if isinstance(val, unicode):
-                safe = '!"#$%&\'()*+,/:;<=>?@[\\]^`{|}~'
-                self.headers[key] = urllib.quote_plus(val.encode('utf-8'), safe)
+        if not getattr(self, '_headers_quoted', False):
+            for key in self.headers:
+                val = self.headers[key]
+                if isinstance(val, six.text_type):
+                    safe = '!"#$%&\'()*+,/:;<=>?@[\\]^`{|}~'
+                    self.headers[key] = quote(val.encode('utf-8'), safe)
+            setattr(self, '_headers_quoted', True)
+
+        self.headers['User-Agent'] = UserAgent
 
         connection._auth_handler.add_auth(self, **kwargs)
 
-        self.headers['User-Agent'] = UserAgent
         # I'm not sure if this is still needed, now that add_auth is
         # setting the content-length for POST requests.
         if 'Content-Length' not in self.headers:
             if 'Transfer-Encoding' not in self.headers or \
                     self.headers['Transfer-Encoding'] != 'chunked':
-                self.headers['Content-Length'] = str(len(self.body))
+                self.headers['Content-Length'] = len(self.body)
 
 
-class HTTPResponse(httplib.HTTPResponse):
+class HTTPResponse(http_client.HTTPResponse):
 
     def __init__(self, *args, **kwargs):
-        httplib.HTTPResponse.__init__(self, *args, **kwargs)
+        http_client.HTTPResponse.__init__(self, *args, **kwargs)
         self._cached_response = ''
 
     def read(self, amt=None):
         """Read the response.
 
         This method does not have the same behavior as
-        httplib.HTTPResponse.read.  Instead, if this method is called with
+        http_client.HTTPResponse.read.  Instead, if this method is called with
         no ``amt`` arg, then the response body will be cached.  Subsequent
         calls to ``read()`` with no args **will return the cached response**.
 
@@ -409,10 +407,10 @@ class HTTPResponse(httplib.HTTPResponse):
             # will return the full body.  Note that this behavior only
             # happens if the amt arg is not specified.
             if not self._cached_response:
-                self._cached_response = httplib.HTTPResponse.read(self)
+                self._cached_response = http_client.HTTPResponse.read(self)
             return self._cached_response
         else:
-            return httplib.HTTPResponse.read(self, amt)
+            return http_client.HTTPResponse.read(self, amt)
 
 
 class AWSAuthConnection(object):
@@ -445,7 +443,7 @@ class AWSAuthConnection(object):
         :type https_connection_factory: list or tuple
         :param https_connection_factory: A pair of an HTTP connection
             factory and the exceptions to catch.  The factory should have
-            a similar interface to L{httplib.HTTPSConnection}.
+            a similar interface to L{http_client.HTTPSConnection}.
 
         :param str proxy: Address/hostname for a proxy server
 
@@ -488,27 +486,30 @@ class AWSAuthConnection(object):
             validate_certs)
         if self.https_validate_certificates and not HAVE_HTTPS_CONNECTION:
             raise BotoClientError(
-                    "SSL server certificate validation is enabled in boto "
-                    "configuration, but Python dependencies required to "
-                    "support this feature are not available. Certificate "
-                    "validation is only supported when running under Python "
-                    "2.6 or later.")
-        self.ca_certificates_file = config.get_value(
-                'Boto', 'ca_certificates_file', DEFAULT_CA_CERTS_FILE)
+                "SSL server certificate validation is enabled in boto "
+                "configuration, but Python dependencies required to "
+                "support this feature are not available. Certificate "
+                "validation is only supported when running under Python "
+                "2.6 or later.")
+        certs_file = config.get_value(
+            'Boto', 'ca_certificates_file', DEFAULT_CA_CERTS_FILE)
+        if certs_file == 'system':
+            certs_file = None
+        self.ca_certificates_file = certs_file
         if port:
             self.port = port
         else:
             self.port = PORTS_BY_SECURITY[is_secure]
 
         self.handle_proxy(proxy, proxy_port, proxy_user, proxy_pass)
-        # define exceptions from httplib that we want to catch and retry
-        self.http_exceptions = (httplib.HTTPException, socket.error,
-                                socket.gaierror, httplib.BadStatusLine)
+        # define exceptions from http_client that we want to catch and retry
+        self.http_exceptions = (http_client.HTTPException, socket.error,
+                                socket.gaierror, http_client.BadStatusLine)
         # define subclasses of the above that are not retryable.
         self.http_unretryable_exceptions = []
         if HAVE_HTTPS_CONNECTION:
             self.http_unretryable_exceptions.append(
-                    https_connection.InvalidCertificateException)
+                https_connection.InvalidCertificateException)
 
         # define values in socket exceptions we don't want to catch
         self.socket_exception_values = (errno.EINTR,)
@@ -524,12 +525,12 @@ class AWSAuthConnection(object):
         self.host = host
         self.path = path
         # if the value passed in for debug
-        if not isinstance(debug, (int, long)):
+        if not isinstance(debug, six.integer_types):
             debug = 0
         self.debug = config.getint('Boto', 'debug', debug)
         self.host_header = None
 
-        # Timeout used to tell httplib how long to wait for socket timeouts.
+        # Timeout used to tell http_client how long to wait for socket timeouts.
         # Default is to leave timeout unchanged, which will in turn result in
         # the socket's default global timeout being used. To specify a
         # timeout, set http_socket_timeout in Boto config. Regardless,
@@ -565,9 +566,10 @@ class AWSAuthConnection(object):
         self._connection = (self.host, self.port, self.is_secure)
         self._last_rs = None
         self._auth_handler = auth.get_auth_handler(
-              host, config, self.provider, self._required_auth_capability())
+            host, config, self.provider, self._required_auth_capability())
         if getattr(self, 'AuthServiceName', None) is not None:
             self.auth_service_name = self.AuthServiceName
+        self.request_hook = None
 
     def __repr__(self):
         return '%s:%s' % (self.__class__.__name__, self.host)
@@ -646,7 +648,7 @@ class AWSAuthConnection(object):
             signature_host = self.host
         else:
             # This unfortunate little hack can be attributed to
-            # a difference in the 2.6 version of httplib.  In old
+            # a difference in the 2.6 version of http_client.  In old
             # versions, it would append ":443" to the hostname sent
             # in the Host header and so we needed to make sure we
             # did the same when calculating the V2 signature.  In 2.6
@@ -666,9 +668,9 @@ class AWSAuthConnection(object):
         self.proxy_pass = proxy_pass
         if 'http_proxy' in os.environ and not self.proxy:
             pattern = re.compile(
-                '(?:http://)?' \
-                '(?:(?P<user>[\w\-\.]+):(?P<pass>.*)@)?' \
-                '(?P<host>[\w\-\.]+)' \
+                '(?:http://)?'
+                '(?:(?P<user>[\w\-\.]+):(?P<pass>.*)@)?'
+                '(?P<host>[\w\-\.]+)'
                 '(?::(?P<port>\d+))?'
             )
             match = pattern.match(os.environ['http_proxy'])
@@ -688,8 +690,8 @@ class AWSAuthConnection(object):
                 self.proxy_pass = config.get_value('Boto', 'proxy_pass', None)
 
         if not self.proxy_port and self.proxy:
-            print "http_proxy environment variable does not specify " \
-                "a port, using default"
+            print("http_proxy environment variable does not specify "
+                  "a port, using default")
             self.proxy_port = self.port
 
         self.no_proxy = os.environ.get('no_proxy', '') or os.environ.get('NO_PROXY', '')
@@ -739,30 +741,30 @@ class AWSAuthConnection(object):
 
         if is_secure:
             boto.log.debug(
-                    'establishing HTTPS connection: host=%s, kwargs=%s',
-                    host, http_connection_kwargs)
+                'establishing HTTPS connection: host=%s, kwargs=%s',
+                host, http_connection_kwargs)
             if self.use_proxy and not self.skip_proxy(host):
                 connection = self.proxy_ssl(host, is_secure and 443 or 80)
             elif self.https_connection_factory:
                 connection = self.https_connection_factory(host)
             elif self.https_validate_certificates and HAVE_HTTPS_CONNECTION:
                 connection = https_connection.CertValidatingHTTPSConnection(
-                        host, ca_certs=self.ca_certificates_file,
-                        **http_connection_kwargs)
+                    host, ca_certs=self.ca_certificates_file,
+                    **http_connection_kwargs)
             else:
-                connection = httplib.HTTPSConnection(host,
-                        **http_connection_kwargs)
+                connection = http_client.HTTPSConnection(
+                    host, **http_connection_kwargs)
         else:
             boto.log.debug('establishing HTTP connection: kwargs=%s' %
-                    http_connection_kwargs)
+                           http_connection_kwargs)
             if self.https_connection_factory:
                 # even though the factory says https, this is too handy
                 # to not be able to allow overriding for http also.
-                connection = self.https_connection_factory(host,
-                    **http_connection_kwargs)
+                connection = self.https_connection_factory(
+                    host, **http_connection_kwargs)
             else:
-                connection = httplib.HTTPConnection(host,
-                    **http_connection_kwargs)
+                connection = http_client.HTTPConnection(
+                    host, **http_connection_kwargs)
         if self.debug > 1:
             connection.set_debuglevel(self.debug)
         # self.connection must be maintained for backwards-compatibility
@@ -783,13 +785,13 @@ class AWSAuthConnection(object):
             host = '%s:%d' % (host, port)
         else:
             host = '%s:%d' % (self.host, self.port)
-        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        try:
-            sock.connect((self.proxy, int(self.proxy_port)))
-            if "timeout" in self.http_connection_kwargs:
-                sock.settimeout(self.http_connection_kwargs["timeout"])
-        except:
-            raise
+        # Seems properly to use timeout for connect too
+        timeout = self.http_connection_kwargs.get("timeout")
+        if timeout is not None:
+            sock = socket.create_connection((self.proxy,
+                                             int(self.proxy_port)), timeout)
+        else:
+            sock = socket.create_connection((self.proxy, int(self.proxy_port)))
         boto.log.debug("Proxy connection: CONNECT %s HTTP/1.0\r\n", host)
         sock.sendall("CONNECT %s HTTP/1.0\r\n" % host)
         sock.sendall("User-Agent: %s\r\n" % UserAgent)
@@ -802,7 +804,7 @@ class AWSAuthConnection(object):
                 sock.sendall("\r\n")
         else:
             sock.sendall("\r\n")
-        resp = httplib.HTTPResponse(sock, strict=True, debuglevel=self.debug)
+        resp = http_client.HTTPResponse(sock, strict=True, debuglevel=self.debug)
         resp.begin()
 
         if resp.status != 200:
@@ -816,12 +818,15 @@ class AWSAuthConnection(object):
         # We can safely close the response, it duped the original socket
         resp.close()
 
-        h = httplib.HTTPConnection(host)
+        h = http_client.HTTPConnection(host)
 
         if self.https_validate_certificates and HAVE_HTTPS_CONNECTION:
-            boto.log.debug("wrapping ssl socket for proxied connection; "
-                           "CA certificate file=%s",
-                           self.ca_certificates_file)
+            msg = "wrapping ssl socket for proxied connection; "
+            if self.ca_certificates_file:
+                msg += "CA certificate file=%s" % self.ca_certificates_file
+            else:
+                msg += "using system provided SSL certs"
+            boto.log.debug(msg)
             key_file = self.http_connection_kwargs.get('key_file', None)
             cert_file = self.http_connection_kwargs.get('cert_file', None)
             sslSock = ssl.wrap_socket(sock, keyfile=key_file,
@@ -832,14 +837,14 @@ class AWSAuthConnection(object):
             hostname = self.host.split(':', 0)[0]
             if not https_connection.ValidateCertificateHostname(cert, hostname):
                 raise https_connection.InvalidCertificateException(
-                        hostname, cert, 'hostname mismatch')
+                    hostname, cert, 'hostname mismatch')
         else:
             # Fallback for old Python without ssl.wrap_socket
-            if hasattr(httplib, 'ssl'):
-                sslSock = httplib.ssl.SSLSocket(sock)
+            if hasattr(http_client, 'ssl'):
+                sslSock = http_client.ssl.SSLSocket(sock)
             else:
                 sslSock = socket.ssl(sock, None, None)
-                sslSock = httplib.FakeSocket(sock, sslSock)
+                sslSock = http_client.FakeSocket(sock, sslSock)
 
         # This is a bit unclean
         h.sock = sslSock
@@ -850,9 +855,24 @@ class AWSAuthConnection(object):
         return path
 
     def get_proxy_auth_header(self):
-        auth = base64.encodestring(self.proxy_user + ':' + self.proxy_pass)
+        auth = encodebytes(self.proxy_user + ':' + self.proxy_pass)
         return {'Proxy-Authorization': 'Basic %s' % auth}
 
+    # For passing proxy information to other connection libraries, e.g. cloudsearch2
+    def get_proxy_url_with_auth(self):
+        if not self.use_proxy:
+            return None
+
+        if self.proxy_user or self.proxy_pass:
+            if self.proxy_pass:
+                login_info = '%s:%s@' % (self.proxy_user, self.proxy_pass)
+            else:
+                login_info = '%s@' % self.proxy_user
+        else:
+            login_info = ''
+
+        return 'http://%s%s:%s' % (login_info, self.proxy, str(self.proxy_port or self.port))
+
     def set_host_header(self, request):
         try:
             request.headers['Host'] = \
@@ -860,6 +880,9 @@ class AWSAuthConnection(object):
         except AttributeError:
             request.headers['Host'] = self.host.split(':', 1)[0]
 
+    def set_request_hook(self, hook):
+        self.request_hook = hook
+
     def _mexe(self, request, sender=None, override_num_retries=None,
               retry_handler=None):
         """
@@ -888,9 +911,16 @@ class AWSAuthConnection(object):
         i = 0
         connection = self.get_http_connection(request.host, request.port,
                                               self.is_secure)
+
+        # Convert body to bytes if needed
+        if not isinstance(request.body, bytes) and hasattr(request.body,
+                                                           'encode'):
+            request.body = request.body.encode('utf-8')
+
         while i <= num_retries:
             # Use binary exponential backoff to desynchronize client requests.
-            next_sleep = random.random() * (2 ** i)
+            next_sleep = min(random.random() * (2 ** i),
+                             boto.config.get('Boto', 'max_retry_delay', 60))
             try:
                 # we now re-sign each request before it is retried
                 boto.log.debug('Token: %s' % self.provider.security_token)
@@ -902,6 +932,8 @@ class AWSAuthConnection(object):
                 if 's3' not in self._required_auth_capability():
                     if not getattr(self, 'anon', False):
                         self.set_host_header(request)
+                boto.log.debug('Final headers: %s' % request.headers)
+                request.start_time = datetime.now()
                 if callable(sender):
                     response = sender(connection, request.method, request.path,
                                       request.body, request.headers)
@@ -909,9 +941,10 @@ class AWSAuthConnection(object):
                     connection.request(request.method, request.path,
                                        request.body, request.headers)
                     response = connection.getresponse()
+                boto.log.debug('Response headers: %s' % response.getheaders())
                 location = response.getheader('location')
                 # -- gross hack --
-                # httplib gets confused with chunked responses to HEAD requests
+                # http_client gets confused with chunked responses to HEAD requests
                 # so I have to fake it out
                 if request.method == 'HEAD' and getattr(response,
                                                         'chunked', False):
@@ -929,6 +962,8 @@ class AWSAuthConnection(object):
                     msg += 'Retrying in %3.1f seconds' % next_sleep
                     boto.log.debug(msg)
                     body = response.read()
+                    if isinstance(body, bytes):
+                        body = body.decode('utf-8')
                 elif response.status < 300 or response.status >= 400 or \
                         not location:
                     # don't return connection to the pool if response contains
@@ -942,10 +977,12 @@ class AWSAuthConnection(object):
                     else:
                         self.put_http_connection(request.host, request.port,
                                                  self.is_secure, connection)
+                    if self.request_hook is not None:
+                        self.request_hook.handle_request_data(request, response)
                     return response
                 else:
                     scheme, request.host, request.path, \
-                        params, query, fragment = urlparse.urlparse(location)
+                        params, query, fragment = urlparse(location)
                     if query:
                         request.path += '?' + query
                     # urlparse can return both host and port in netloc, so if
@@ -960,20 +997,20 @@ class AWSAuthConnection(object):
                                                           scheme == 'https')
                     response = None
                     continue
-            except PleaseRetryException, e:
+            except PleaseRetryException as e:
                 boto.log.debug('encountered a retry exception: %s' % e)
                 connection = self.new_http_connection(request.host, request.port,
                                                       self.is_secure)
                 response = e.response
-            except self.http_exceptions, e:
+            except self.http_exceptions as e:
                 for unretryable in self.http_unretryable_exceptions:
                     if isinstance(e, unretryable):
                         boto.log.debug(
                             'encountered unretryable %s exception, re-raising' %
                             e.__class__.__name__)
                         raise
-                boto.log.debug('encountered %s exception, reconnecting' % \
-                                  e.__class__.__name__)
+                boto.log.debug('encountered %s exception, reconnecting' %
+                               e.__class__.__name__)
                 connection = self.new_http_connection(request.host, request.port,
                                                       self.is_secure)
             time.sleep(next_sleep)
@@ -982,6 +1019,8 @@ class AWSAuthConnection(object):
         # and stil haven't succeeded.  So, if we have a response object,
         # use it to raise an exception.
         # Otherwise, raise the exception that must have already happened.
+        if self.request_hook is not None:
+            self.request_hook.handle_request_data(request, response, error=True)
         if response:
             raise BotoServerError(response.status, response.reason, body)
         elif e:
@@ -1003,8 +1042,7 @@ class AWSAuthConnection(object):
             headers = {}
         else:
             headers = headers.copy()
-        if (self.host_header and
-            not boto.utils.find_matching_headers('host', headers)):
+        if self.host_header and not boto.utils.find_matching_headers('host', headers):
             headers['host'] = self.host_header
         host = host or self.host
         if self.use_proxy:
@@ -1047,14 +1085,15 @@ class AWSQueryConnection(AWSAuthConnection):
                  proxy_user=None, proxy_pass=None, host=None, debug=0,
                  https_connection_factory=None, path='/', security_token=None,
                  validate_certs=True, profile_name=None):
-        super(AWSQueryConnection, self).__init__(host, aws_access_key_id,
-                                   aws_secret_access_key,
-                                   is_secure, port, proxy,
-                                   proxy_port, proxy_user, proxy_pass,
-                                   debug, https_connection_factory, path,
-                                   security_token=security_token,
-                                   validate_certs=validate_certs,
-                                   profile_name=profile_name)
+        super(AWSQueryConnection, self).__init__(
+            host, aws_access_key_id,
+            aws_secret_access_key,
+            is_secure, port, proxy,
+            proxy_port, proxy_user, proxy_pass,
+            debug, https_connection_factory, path,
+            security_token=security_token,
+            validate_certs=validate_certs,
+            profile_name=profile_name)
 
     def _required_auth_capability(self):
         return []
@@ -1073,7 +1112,7 @@ class AWSQueryConnection(AWSAuthConnection):
         return self._mexe(http_request)
 
     def build_list_params(self, params, items, label):
-        if isinstance(items, basestring):
+        if isinstance(items, six.string_types):
             items = [items]
         for i in range(1, len(items) + 1):
             params['%s.%d' % (label, i)] = items[i - 1]
@@ -1133,6 +1172,8 @@ class AWSQueryConnection(AWSAuthConnection):
         elif response.status == 200:
             rs = ResultSet(markers)
             h = boto.handler.XmlHandler(rs, parent)
+            if isinstance(body, six.text_type):
+                body = body.encode('utf-8')
             xml.sax.parseString(body, h)
             return rs
         else:
@@ -1153,6 +1194,8 @@ class AWSQueryConnection(AWSAuthConnection):
         elif response.status == 200:
             obj = cls(parent)
             h = boto.handler.XmlHandler(obj, parent)
+            if isinstance(body, six.text_type):
+                body = body.encode('utf-8')
             xml.sax.parseString(body, h)
             return obj
         else:
