@@ -30,9 +30,9 @@ from boto.ec2.instanceinfo import InstanceInfo
 from boto.ec2.elb.loadbalancer import LoadBalancer, LoadBalancerZones
 from boto.ec2.elb.instancestate import InstanceState
 from boto.ec2.elb.healthcheck import HealthCheck
-from boto.ec2.elb.listelement import ListElement
 from boto.regioninfo import RegionInfo, get_regions, load_regions
 import boto
+from boto.compat import six
 
 RegionData = load_regions().get('elasticloadbalancing', {})
 
@@ -68,8 +68,9 @@ class ELBConnection(AWSQueryConnection):
 
     APIVersion = boto.config.get('Boto', 'elb_version', '2012-06-01')
     DefaultRegionName = boto.config.get('Boto', 'elb_region_name', 'us-east-1')
-    DefaultRegionEndpoint = boto.config.get('Boto', 'elb_region_endpoint',
-                                            'elasticloadbalancing.us-east-1.amazonaws.com')
+    DefaultRegionEndpoint = boto.config.get(
+        'Boto', 'elb_region_endpoint',
+        'elasticloadbalancing.us-east-1.amazonaws.com')
 
     def __init__(self, aws_access_key_id=None, aws_secret_access_key=None,
                  is_secure=True, port=None, proxy=None, proxy_port=None,
@@ -87,31 +88,37 @@ class ELBConnection(AWSQueryConnection):
                                 self.DefaultRegionEndpoint)
         self.region = region
         super(ELBConnection, self).__init__(aws_access_key_id,
-                                    aws_secret_access_key,
-                                    is_secure, port, proxy, proxy_port,
-                                    proxy_user, proxy_pass,
-                                    self.region.endpoint, debug,
-                                    https_connection_factory, path,
-                                    security_token,
-                                    validate_certs=validate_certs,
-                                    profile_name=profile_name)
+                                            aws_secret_access_key,
+                                            is_secure, port, proxy, proxy_port,
+                                            proxy_user, proxy_pass,
+                                            self.region.endpoint, debug,
+                                            https_connection_factory, path,
+                                            security_token,
+                                            validate_certs=validate_certs,
+                                            profile_name=profile_name)
 
     def _required_auth_capability(self):
         return ['ec2']
 
     def build_list_params(self, params, items, label):
-        if isinstance(items, basestring):
+        if isinstance(items, six.string_types):
             items = [items]
         for index, item in enumerate(items):
             params[label % (index + 1)] = item
 
-    def get_all_load_balancers(self, load_balancer_names=None):
+    def get_all_load_balancers(self, load_balancer_names=None, marker=None):
         """
         Retrieve all load balancers associated with your account.
 
         :type load_balancer_names: list
         :keyword load_balancer_names: An optional list of load balancer names.
 
+        :type marker: string
+        :param marker: Use this only when paginating results and only
+            in follow-up request after you've received a response
+            where the results are truncated.  Set this to the value of
+            the Marker element in the response you just received.
+
         :rtype: :py:class:`boto.resultset.ResultSet`
         :return: A ResultSet containing instances of
             :class:`boto.ec2.elb.loadbalancer.LoadBalancer`
@@ -120,11 +127,16 @@ class ELBConnection(AWSQueryConnection):
         if load_balancer_names:
             self.build_list_params(params, load_balancer_names,
                                    'LoadBalancerNames.member.%d')
+
+        if marker:
+            params['Marker'] = marker
+
         return self.get_list('DescribeLoadBalancers', params,
                              [('member', LoadBalancer)])
 
     def create_load_balancer(self, name, zones, listeners=None, subnets=None,
-        security_groups=None, scheme='internet-facing', complex_listeners=None):
+                             security_groups=None, scheme='internet-facing',
+                             complex_listeners=None):
         """
         Create a new load balancer for your account. By default the load
         balancer will be created in EC2. To create a load balancer inside a
@@ -170,13 +182,14 @@ class ELBConnection(AWSQueryConnection):
 
         :type complex_listeners: List of tuples
         :param complex_listeners: Each tuple contains four or five values,
-            (LoadBalancerPortNumber, InstancePortNumber, Protocol, InstanceProtocol,
-            SSLCertificateId).
+            (LoadBalancerPortNumber, InstancePortNumber, Protocol,
+             InstanceProtocol, SSLCertificateId).
 
             Where:
                 - LoadBalancerPortNumber and InstancePortNumber are integer
                   values between 1 and 65535
-                - Protocol and InstanceProtocol is a string containing either 'TCP',
+                - Protocol and InstanceProtocol is a string containing
+                  either 'TCP',
                   'SSL', 'HTTP', or 'HTTPS'
                 - SSLCertificateId is the ARN of an SSL certificate loaded into
                   AWS IAM
@@ -224,7 +237,7 @@ class ELBConnection(AWSQueryConnection):
 
         if security_groups:
             self.build_list_params(params, security_groups,
-                                    'SecurityGroups.member.%d')
+                                   'SecurityGroups.member.%d')
 
         load_balancer = self.get_object('CreateLoadBalancer',
                                         params, LoadBalancer)
@@ -235,7 +248,8 @@ class ELBConnection(AWSQueryConnection):
         load_balancer.security_groups = security_groups
         return load_balancer
 
-    def create_load_balancer_listeners(self, name, listeners=None, complex_listeners=None):
+    def create_load_balancer_listeners(self, name, listeners=None,
+                                       complex_listeners=None):
         """
         Creates a Listener (or group of listeners) for an existing
         Load Balancer
@@ -254,13 +268,14 @@ class ELBConnection(AWSQueryConnection):
 
         :type complex_listeners: List of tuples
         :param complex_listeners: Each tuple contains four or five values,
-            (LoadBalancerPortNumber, InstancePortNumber, Protocol, InstanceProtocol,
-            SSLCertificateId).
+            (LoadBalancerPortNumber, InstancePortNumber, Protocol,
+             InstanceProtocol, SSLCertificateId).
 
             Where:
                 - LoadBalancerPortNumber and InstancePortNumber are integer
                   values between 1 and 65535
-                - Protocol and InstanceProtocol is a string containing either 'TCP',
+                - Protocol and InstanceProtocol is a string containing
+                  either 'TCP',
                   'SSL', 'HTTP', or 'HTTPS'
                 - SSLCertificateId is the ARN of an SSL certificate loaded into
                   AWS IAM
@@ -347,7 +362,7 @@ class ELBConnection(AWSQueryConnection):
         self.build_list_params(params, zones_to_add,
                                'AvailabilityZones.member.%d')
         obj = self.get_object('EnableAvailabilityZonesForLoadBalancer',
-                               params, LoadBalancerZones)
+                              params, LoadBalancerZones)
         return obj.zones
 
     def disable_availability_zones(self, load_balancer_name, zones_to_remove):
@@ -372,7 +387,7 @@ class ELBConnection(AWSQueryConnection):
         self.build_list_params(params, zones_to_remove,
                                'AvailabilityZones.member.%d')
         obj = self.get_object('DisableAvailabilityZonesForLoadBalancer',
-                               params, LoadBalancerZones)
+                              params, LoadBalancerZones)
         return obj.zones
 
     def modify_lb_attribute(self, load_balancer_name, attribute, value):
@@ -385,6 +400,7 @@ class ELBConnection(AWSQueryConnection):
         :param attribute: The attribute you wish to change.
 
         * crossZoneLoadBalancing - Boolean (true)
+        * connectingSettings - :py:class:`ConnectionSettingAttribute` instance
         * accessLog - :py:class:`AccessLogAttribute` instance
         * connectionDraining - :py:class:`ConnectionDrainingAttribute` instance
 
@@ -421,6 +437,9 @@ class ELBConnection(AWSQueryConnection):
                 value.enabled and 'true' or 'false'
             params['LoadBalancerAttributes.ConnectionDraining.Timeout'] = \
                 value.timeout
+        elif attribute.lower() == 'connectingsettings':
+            params['LoadBalancerAttributes.ConnectionSettings.IdleTimeout'] = \
+                value.idle_timeout
         else:
             raise ValueError('InvalidAttribute', attribute)
         return self.get_status('ModifyLoadBalancerAttributes', params,
@@ -453,7 +472,9 @@ class ELBConnection(AWSQueryConnection):
 
           * accessLog - :py:class:`AccessLogAttribute` instance
           * crossZoneLoadBalancing - Boolean
-          * connectionDraining - :py:class:`ConnectionDrainingAttribute` instance
+          * connectingSettings - :py:class:`ConnectionSettingAttribute` instance
+          * connectionDraining - :py:class:`ConnectionDrainingAttribute`
+            instance
 
         :rtype: Attribute dependent
         :return: The new value for the attribute
@@ -465,6 +486,8 @@ class ELBConnection(AWSQueryConnection):
             return attributes.cross_zone_load_balancing.enabled
         if attribute.lower() == 'connectiondraining':
             return attributes.connection_draining
+        if attribute.lower() == 'connectingsettings':
+            return attributes.connecting_settings
         return None
 
     def register_instances(self, load_balancer_name, instances):
@@ -613,17 +636,19 @@ class ELBConnection(AWSQueryConnection):
             params['CookieExpirationPeriod'] = cookie_expiration_period
         return self.get_status('CreateLBCookieStickinessPolicy', params)
 
-    def create_lb_policy(self, lb_name, policy_name, policy_type, policy_attributes):
+    def create_lb_policy(self, lb_name, policy_name, policy_type,
+                         policy_attributes):
         """
-        Creates a new policy that contains the necessary attributes depending on
-        the policy type. Policies are settings that are saved for your load
-        balancer and that can be applied to the front-end listener, or
-        the back-end application server.
+        Creates a new policy that contains the necessary attributes
+        depending on the policy type. Policies are settings that are
+        saved for your load balancer and that can be applied to the
+        front-end listener, or the back-end application server.
+
         """
         params = {'LoadBalancerName': lb_name,
                   'PolicyName': policy_name,
                   'PolicyTypeName': policy_type}
-        for index, (name, value) in enumerate(policy_attributes.iteritems(), 1):
+        for index, (name, value) in enumerate(six.iteritems(policy_attributes), 1):
             params['PolicyAttributes.member.%d.AttributeName' % index] = name
             params['PolicyAttributes.member.%d.AttributeValue' % index] = value
         else:
@@ -653,7 +678,8 @@ class ELBConnection(AWSQueryConnection):
             params['PolicyNames'] = ''
         return self.get_status('SetLoadBalancerPoliciesOfListener', params)
 
-    def set_lb_policies_of_backend_server(self, lb_name, instance_port, policies):
+    def set_lb_policies_of_backend_server(self, lb_name, instance_port,
+                                          policies):
         """
         Replaces the current set of policies associated with a port on which
         the back-end server is listening with a new set of policies.
@@ -664,7 +690,8 @@ class ELBConnection(AWSQueryConnection):
             self.build_list_params(params, policies, 'PolicyNames.member.%d')
         else:
             params['PolicyNames'] = ''
-        return self.get_status('SetLoadBalancerPoliciesForBackendServer', params)
+        return self.get_status('SetLoadBalancerPoliciesForBackendServer',
+                               params)
 
     def apply_security_groups_to_lb(self, name, security_groups):
         """
