@@ -21,16 +21,37 @@
 # IN THE SOFTWARE.
 import boto
 import boto.jsonresponse
-from boto.compat import json
+from boto.compat import json, six
 from boto.resultset import ResultSet
 from boto.iam.summarymap import SummaryMap
 from boto.connection import AWSQueryConnection
 
-
-ASSUME_ROLE_POLICY_DOCUMENT = json.dumps({
-    'Statement': [{'Principal': {'Service': ['ec2.amazonaws.com']},
-                   'Effect': 'Allow',
-                   'Action': ['sts:AssumeRole']}]})
+DEFAULT_POLICY_DOCUMENTS = {
+    'default': {
+        'Statement': [
+            {
+                'Principal': {
+                    'Service': ['ec2.amazonaws.com']
+                },
+                'Effect': 'Allow',
+                'Action': ['sts:AssumeRole']
+            }
+        ]
+    },
+    'amazonaws.com.cn': {
+        'Statement': [
+            {
+                'Principal': {
+                    'Service': ['ec2.amazonaws.com.cn']
+                },
+                'Effect': 'Allow',
+                'Action': ['sts:AssumeRole']
+            }
+        ]
+    },
+}
+# For backward-compatibility, we'll preserve this here.
+ASSUME_ROLE_POLICY_DOCUMENT = json.dumps(DEFAULT_POLICY_DOCUMENTS['default'])
 
 
 class IAMConnection(AWSQueryConnection):
@@ -40,16 +61,16 @@ class IAMConnection(AWSQueryConnection):
     def __init__(self, aws_access_key_id=None, aws_secret_access_key=None,
                  is_secure=True, port=None, proxy=None, proxy_port=None,
                  proxy_user=None, proxy_pass=None, host='iam.amazonaws.com',
-                 debug=0, https_connection_factory=None, path='/', 
+                 debug=0, https_connection_factory=None, path='/',
                  security_token=None, validate_certs=True, profile_name=None):
         super(IAMConnection, self).__init__(aws_access_key_id,
-                                    aws_secret_access_key,
-                                    is_secure, port, proxy,
-                                    proxy_port, proxy_user, proxy_pass,
-                                    host, debug, https_connection_factory,
-                                    path, security_token,
-                                    validate_certs=validate_certs,
-                                    profile_name=profile_name)
+                                            aws_secret_access_key,
+                                            is_secure, port, proxy,
+                                            proxy_port, proxy_user, proxy_pass,
+                                            host, debug, https_connection_factory,
+                                            path, security_token,
+                                            validate_certs=validate_certs,
+                                            profile_name=profile_name)
 
     def _required_auth_capability(self):
         return ['hmac-v4']
@@ -679,7 +700,7 @@ class IAMConnection(AWSQueryConnection):
     #
 
     def list_server_certs(self, path_prefix='/',
-                             marker=None, max_items=None):
+                          marker=None, max_items=None):
         """
         Lists the server certificates that have the specified path prefix.
         If none exist, the action returns an empty list.
@@ -1006,13 +1027,35 @@ class IAMConnection(AWSQueryConnection):
         :param service: Default service to go to in the console.
         """
         alias = self.get_account_alias()
+
         if not alias:
             raise Exception('No alias associated with this account.  Please use iam.create_account_alias() first.')
 
+        resp = alias.get('list_account_aliases_response', {})
+        result = resp.get('list_account_aliases_result', {})
+        aliases = result.get('account_aliases', [])
+
+        if not len(aliases):
+            raise Exception('No alias associated with this account.  Please use iam.create_account_alias() first.')
+
+        # We'll just use the first one we find.
+        alias = aliases[0]
+
         if self.host == 'iam.us-gov.amazonaws.com':
-            return "https://%s.signin.amazonaws-us-gov.com/console/%s" % (alias, service)
+            return "https://%s.signin.amazonaws-us-gov.com/console/%s" % (
+                alias,
+                service
+            )
+        elif self.host.endswith('amazonaws.com.cn'):
+            return "https://%s.signin.amazonaws.cn/console/%s" % (
+                alias,
+                service
+            )
         else:
-            return "https://%s.signin.aws.amazon.com/console/%s" % (alias, service)
+            return "https://%s.signin.aws.amazon.com/console/%s" % (
+                alias,
+                service
+            )
 
     def get_account_summary(self):
         """
@@ -1059,6 +1102,30 @@ class IAMConnection(AWSQueryConnection):
             params['Path'] = path
         return self.get_response('CreateInstanceProfile', params)
 
+    def _build_policy(self, assume_role_policy_document=None):
+        if assume_role_policy_document is not None:
+            if isinstance(assume_role_policy_document, six.string_types):
+                # Historically, they had to pass a string. If it's a string,
+                # assume the user has already handled it.
+                return assume_role_policy_document
+        else:
+
+            for tld, policy in DEFAULT_POLICY_DOCUMENTS.items():
+                if tld is 'default':
+                    # Skip the default. We'll fall back to it if we don't find
+                    # anything.
+                    continue
+
+                if self.host and self.host.endswith(tld):
+                    assume_role_policy_document = policy
+                    break
+
+            if not assume_role_policy_document:
+                assume_role_policy_document = DEFAULT_POLICY_DOCUMENTS['default']
+
+        # Dump the policy (either user-supplied ``dict`` or one of the defaults)
+        return json.dumps(assume_role_policy_document)
+
     def create_role(self, role_name, assume_role_policy_document=None, path=None):
         """
         Creates a new role for your AWS account.
@@ -1070,21 +1137,19 @@ class IAMConnection(AWSQueryConnection):
         :type role_name: string
         :param role_name: Name of the role to create.
 
-        :type assume_role_policy_document: string
+        :type assume_role_policy_document: ``string`` or ``dict``
         :param assume_role_policy_document: The policy that grants an entity
             permission to assume the role.
 
         :type path: string
-        :param path: The path to the instance profile.
+        :param path: The path to the role.
         """
-        params = {'RoleName': role_name}
-        if assume_role_policy_document is None:
-            # This is the only valid assume_role_policy_document currently, so
-            # this is used as a default value if no assume_role_policy_document
-            # is provided.
-            params['AssumeRolePolicyDocument'] =  ASSUME_ROLE_POLICY_DOCUMENT
-        else:
-            params['AssumeRolePolicyDocument'] =  assume_role_policy_document
+        params = {
+            'RoleName': role_name,
+            'AssumeRolePolicyDocument': self._build_policy(
+                assume_role_policy_document
+            ),
+        }
         if path is not None:
             params['Path'] = path
         return self.get_response('CreateRole', params)
@@ -1134,8 +1199,8 @@ class IAMConnection(AWSQueryConnection):
         :param instance_profile_name: Name of the instance profile to get
             information about.
         """
-        return self.get_response('GetInstanceProfile', {'InstanceProfileName':
-                                                       instance_profile_name})
+        return self.get_response('GetInstanceProfile',
+                                 {'InstanceProfileName': instance_profile_name})
 
     def get_role(self, role_name):
         """
@@ -1375,7 +1440,7 @@ class IAMConnection(AWSQueryConnection):
         Lists the SAML providers in the account.
         This operation requires `Signature Version 4`_.
         """
-        return self.get_response('ListSAMLProviders', {})
+        return self.get_response('ListSAMLProviders', {}, list_marker='SAMLProviderList')
 
     def get_saml_provider(self, saml_provider_arn):
         """
@@ -1388,7 +1453,7 @@ class IAMConnection(AWSQueryConnection):
             provider to get information about.
 
         """
-        params = {'SAMLProviderArn': saml_provider_arn }
+        params = {'SAMLProviderArn': saml_provider_arn}
         return self.get_response('GetSAMLProvider', params)
 
     def update_saml_provider(self, saml_provider_arn, saml_metadata_document):
@@ -1431,5 +1496,51 @@ class IAMConnection(AWSQueryConnection):
             provider to delete.
 
         """
-        params = {'SAMLProviderArn': saml_provider_arn }
+        params = {'SAMLProviderArn': saml_provider_arn}
         return self.get_response('DeleteSAMLProvider', params)
+
+    #
+    # IAM Reports
+    #
+
+    def generate_credential_report(self):
+        """
+        Generates a credential report for an account
+
+        A new credential report can only be generated every 4 hours. If one
+        hasn't been generated in the last 4 hours then get_credential_report
+        will error when called
+        """
+        params = {}
+        return self.get_response('GenerateCredentialReport', params)
+
+    def get_credential_report(self):
+        """
+        Retrieves a credential report for an account
+
+        A report must have been generated in the last 4 hours to succeed.
+        The report is returned as a base64 encoded blob within the response.
+        """
+        params = {}
+        return self.get_response('GetCredentialReport', params)
+
+    def create_virtual_mfa_device(self, path, device_name):
+        """
+        Creates a new virtual MFA device for the AWS account.
+
+        After creating the virtual MFA, use enable-mfa-device to
+        attach the MFA device to an IAM user.
+
+        :type path: string
+        :param path: The path for the virtual MFA device.
+
+        :type device_name: string
+        :param device_name: The name of the virtual MFA device.
+            Used with path to uniquely identify a virtual MFA device.
+
+        """
+        params = {
+            'Path': path,
+            'VirtualMFADeviceName': device_name
+        }
+        return self.get_response('CreateVirtualMFADevice', params)
