@@ -20,14 +20,13 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 # IN THE SOFTWARE.
 #
-import ConfigParser
 import os
 import re
-import StringIO
 import warnings
 
 import boto
-from boto.compat import expanduser
+
+from boto.compat import expanduser, ConfigParser, StringIO
 
 
 # By default we use two locations for the boto configurations,
@@ -50,13 +49,13 @@ elif 'BOTO_PATH' in os.environ:
         BotoConfigLocations.append(expanduser(path))
 
 
-class Config(ConfigParser.SafeConfigParser):
+class Config(ConfigParser):
 
     def __init__(self, path=None, fp=None, do_load=True):
         # We don't use ``super`` here, because ``ConfigParser`` still uses
         # old-style classes.
-        ConfigParser.SafeConfigParser.__init__(self, {'working_dir' : '/mnt/pyami',
-                                                      'debug' : '0'})
+        ConfigParser.__init__(self, {'working_dir': '/mnt/pyami',
+                                         'debug': '0'})
         if do_load:
             if path:
                 self.load_from_path(path)
@@ -73,7 +72,7 @@ class Config(ConfigParser.SafeConfigParser):
 
     def load_credential_file(self, path):
         """Load a credential file as is setup like the Java utilities"""
-        c_data = StringIO.StringIO()
+        c_data = StringIO()
         c_data.write("[Credentials]\n")
         for line in open(path, "r").readlines():
             c_data.write(line.replace("AWSAccessKeyId", "aws_access_key_id").replace("AWSSecretKey", "aws_secret_access_key"))
@@ -96,7 +95,7 @@ class Config(ConfigParser.SafeConfigParser):
         Replace any previous value.  If the path doesn't exist, create it.
         Also add the option the the in-memory config.
         """
-        config = ConfigParser.SafeConfigParser()
+        config = ConfigParser()
         config.read(path)
         if not config.has_section(section):
             config.add_section(section)
@@ -140,21 +139,21 @@ class Config(ConfigParser.SafeConfigParser):
 
     def get(self, section, name, default=None):
         try:
-            val = ConfigParser.SafeConfigParser.get(self, section, name)
+            val = ConfigParser.get(self, section, name)
         except:
             val = default
         return val
 
     def getint(self, section, name, default=0):
         try:
-            val = ConfigParser.SafeConfigParser.getint(self, section, name)
+            val = ConfigParser.getint(self, section, name)
         except:
             val = int(default)
         return val
 
     def getfloat(self, section, name, default=0.0):
         try:
-            val = ConfigParser.SafeConfigParser.getfloat(self, section, name)
+            val = ConfigParser.getfloat(self, section, name)
         except:
             val = float(default)
         return val
@@ -177,13 +176,13 @@ class Config(ConfigParser.SafeConfigParser):
             self.set(section, name, 'false')
 
     def dump(self):
-        s = StringIO.StringIO()
+        s = StringIO()
         self.write(s)
-        print s.getvalue()
+        print(s.getvalue())
 
     def dump_safe(self, fp=None):
         if not fp:
-            fp = StringIO.StringIO()
+            fp = StringIO()
         for section in self.sections():
             fp.write('[%s]\n' % section)
             for option in self.options(section):
