@@ -20,19 +20,19 @@
 # WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 # IN THE SOFTWARE.
-
-from __future__ import with_statement
+import email.utils
 import errno
+import hashlib
 import mimetypes
 import os
 import re
-import rfc822
-import StringIO
 import base64
 import binascii
 import math
-import urllib
+from hashlib import md5
 import boto.utils
+from boto.compat import BytesIO, six, urllib, encodebytes
+
 from boto.exception import BotoClientError
 from boto.exception import StorageDataError
 from boto.exception import PleaseRetryException
@@ -40,13 +40,9 @@ from boto.provider import Provider
 from boto.s3.keyfile import KeyFile
 from boto.s3.user import User
 from boto import UserAgent
-from boto.utils import compute_md5
+from boto.utils import compute_md5, compute_hash
 from boto.utils import find_matching_headers
 from boto.utils import merge_headers_by_name
-try:
-    from hashlib import md5
-except ImportError:
-    from md5 import md5
 
 
 class Key(object):
@@ -93,10 +89,16 @@ class Key(object):
     # x-amz-meta).
     base_user_settable_fields = set(["cache-control", "content-disposition",
                                     "content-encoding", "content-language",
-                                    "content-md5", "content-type"])
+                                    "content-md5", "content-type",
+                                     "x-robots-tag", "expires"])
     _underscore_base_user_settable_fields = set()
     for f in base_user_settable_fields:
       _underscore_base_user_settable_fields.add(f.replace('-', '_'))
+    # Metadata fields, whether user-settable or not, other than custom
+    # metadata fields (i.e., those beginning with a provider specific prefix
+    # like x-amz-meta).
+    base_fields = (base_user_settable_fields |
+                   set(["last-modified", "content-length", "date", "etag"]))
 
 
 
@@ -114,7 +116,7 @@ class Key(object):
         self.is_latest = False
         self.last_modified = None
         self.owner = None
-        self.storage_class = 'STANDARD'
+        self._storage_class = None
         self.path = None
         self.resp = None
         self.mode = None
@@ -134,9 +136,15 @@ class Key(object):
 
     def __repr__(self):
         if self.bucket:
-            return '<Key: %s,%s>' % (self.bucket.name, self.name)
+            name = u'<Key: %s,%s>' % (self.bucket.name, self.name)
         else:
-            return '<Key: None,%s>' % self.name
+            name = u'<Key: None,%s>' % self.name
+
+        # Encode to bytes for Python 2 to prevent display decoding issues
+        if not isinstance(name, str):
+            name = name.encode('utf-8')
+
+        return name
 
     def __iter__(self):
         return self
@@ -170,23 +178,46 @@ class Key(object):
 
     def _get_base64md5(self):
         if 'md5' in self.local_hashes and self.local_hashes['md5']:
-            return binascii.b2a_base64(self.local_hashes['md5']).rstrip('\n')
+            md5 = self.local_hashes['md5']
+            if not isinstance(md5, bytes):
+                md5 = md5.encode('utf-8')
+            return binascii.b2a_base64(md5).decode('utf-8').rstrip('\n')
 
     def _set_base64md5(self, value):
         if value:
+            if not isinstance(value, six.string_types):
+                value = value.decode('utf-8')
             self.local_hashes['md5'] = binascii.a2b_base64(value)
         elif 'md5' in self.local_hashes:
             del self.local_hashes['md5']
 
     base64md5 = property(_get_base64md5, _set_base64md5);
 
+    def _get_storage_class(self):
+        if self._storage_class is None and self.bucket:
+            # Attempt to fetch storage class
+            list_items = list(self.bucket.list(self.name.encode('utf-8')))
+            if len(list_items) and getattr(list_items[0], '_storage_class',
+                                           None):
+                self._storage_class = list_items[0]._storage_class
+            else:
+                # Key is not yet saved? Just use default...
+                self._storage_class = 'STANDARD'
+
+        return self._storage_class
+
+    def _set_storage_class(self, value):
+        self._storage_class = value
+
+    storage_class = property(_get_storage_class, _set_storage_class)
+
     def get_md5_from_hexdigest(self, md5_hexdigest):
         """
         A utility function to create the 2-tuple (md5hexdigest, base64md5)
         from just having a precalculated md5_hexdigest.
         """
         digest = binascii.unhexlify(md5_hexdigest)
-        base64md5 = base64.encodestring(digest)
+        base64md5 = encodebytes(digest)
         if base64md5[-1] == '\n':
             base64md5 = base64md5[0:-1]
         return (md5_hexdigest, base64md5)
@@ -216,7 +247,8 @@ class Key(object):
             self.delete_marker = False
 
     def handle_restore_headers(self, response):
-        header = response.getheader('x-amz-restore')
+        provider = self.bucket.connection.provider
+        header = response.getheader(provider.restore_header)
         if header is None:
             return
         parts = header.split(',', 1)
@@ -257,7 +289,7 @@ class Key(object):
             with the stored object in the response.  See
             http://goo.gl/EWOPb for details.
         """
-        if self.resp == None:
+        if self.resp is None:
             self.mode = 'r'
 
             provider = self.bucket.connection.provider
@@ -282,22 +314,11 @@ class Key(object):
                 elif name.lower() == 'content-range':
                     end_range = re.sub('.*/(.*)', '\\1', value)
                     self.size = int(end_range)
-                elif name.lower() == 'etag':
-                    self.etag = value
-                elif name.lower() == 'content-type':
-                    self.content_type = value
-                elif name.lower() == 'content-encoding':
-                    self.content_encoding = value
-                elif name.lower() == 'content-language':
-                    self.content_language = value
-                elif name.lower() == 'last-modified':
-                    self.last_modified = value
-                elif name.lower() == 'cache-control':
-                    self.cache_control = value
-                elif name.lower() == 'content-disposition':
-                    self.content_disposition = value
+                elif name.lower() in Key.base_fields:
+                    self.__dict__[name.lower().replace('-', '_')] = value
             self.handle_version_headers(self.resp)
             self.handle_encryption_headers(self.resp)
+            self.handle_restore_headers(self.resp)
             self.handle_addl_headers(self.resp.getheaders())
 
     def open_write(self, headers=None, override_num_retries=None):
@@ -368,6 +389,9 @@ class Key(object):
             raise StopIteration
         return data
 
+    # Python 3 iterator support
+    __next__ = next
+
     def read(self, size=0):
         self.open_read()
         if size == 0:
@@ -531,25 +555,27 @@ class Key(object):
             self.metadata['Content-MD5'] = value
         else:
             self.metadata[name] = value
+        if name.lower() in Key.base_user_settable_fields:
+            self.__dict__[name.lower().replace('-', '_')] = value
 
     def update_metadata(self, d):
         self.metadata.update(d)
 
     # convenience methods for setting/getting ACL
     def set_acl(self, acl_str, headers=None):
-        if self.bucket != None:
+        if self.bucket is not None:
             self.bucket.set_acl(acl_str, self.name, headers=headers)
 
     def get_acl(self, headers=None):
-        if self.bucket != None:
+        if self.bucket is not None:
             return self.bucket.get_acl(self.name, headers=headers)
 
     def get_xml_acl(self, headers=None):
-        if self.bucket != None:
+        if self.bucket is not None:
             return self.bucket.get_xml_acl(self.name, headers=headers)
 
     def set_xml_acl(self, acl_str, headers=None):
-        if self.bucket != None:
+        if self.bucket is not None:
             return self.bucket.set_xml_acl(acl_str, self.name, headers=headers)
 
     def set_canned_acl(self, acl_str, headers=None):
@@ -798,6 +824,10 @@ class Key(object):
                 chunk = fp.read(bytes_togo)
             else:
                 chunk = fp.read(self.BufferSize)
+
+            if not isinstance(chunk, bytes):
+                chunk = chunk.encode('utf-8')
+
             if spos is None:
                 # read at least something from a non-seekable fp.
                 self.read_from_stream = True
@@ -826,6 +856,9 @@ class Key(object):
                 else:
                     chunk = fp.read(self.BufferSize)
 
+                if not isinstance(chunk, bytes):
+                    chunk = chunk.encode('utf-8')
+
             self.size = data_len
 
             for alg in digesters:
@@ -858,7 +891,9 @@ class Key(object):
         for header in find_matching_headers('User-Agent', headers):
             del headers[header]
         headers['User-Agent'] = UserAgent
-        if self.storage_class != 'STANDARD':
+        # If storage_class is None, then a user has not explicitly requested
+        # a storage class, so we can assume STANDARD here
+        if self._storage_class not in [None, 'STANDARD']:
             headers[provider.storage_class_header] = self.storage_class
         if find_matching_headers('Content-Encoding', headers):
             self.content_encoding = merge_headers_by_name(
@@ -881,7 +916,7 @@ class Key(object):
                     'Content-Type', headers)
         elif self.path:
             self.content_type = mimetypes.guess_type(self.path)[0]
-            if self.content_type == None:
+            if self.content_type is None:
                 self.content_type = self.DefaultContentType
             headers['Content-Type'] = self.content_type
         else:
@@ -894,6 +929,15 @@ class Key(object):
             #    headers['Trailer'] = "Content-MD5"
         else:
             headers['Content-Length'] = str(self.size)
+        # This is terrible. We need a SHA256 of the body for SigV4, but to do
+        # the chunked ``sender`` behavior above, the ``fp`` isn't available to
+        # the auth mechanism (because closures). Detect if it's SigV4 & embelish
+        # while we can before the auth calculations occur.
+        if 'hmac-v4-s3' in self.bucket.connection._required_auth_capability():
+            kwargs = {'fp': fp, 'hash_algorithm': hashlib.sha256}
+            if size is not None:
+                kwargs['size'] = size
+            headers['_sha256'] = compute_hash(**kwargs)[0]
         headers['Expect'] = '100-Continue'
         headers = boto.utils.merge_meta(headers, self.metadata, provider)
         resp = self.bucket.connection.make_request(
@@ -921,10 +965,20 @@ class Key(object):
 
         if 200 <= response.status <= 299:
             self.etag = response.getheader('etag')
+            md5 = self.md5
+            if isinstance(md5, bytes):
+                md5 = md5.decode('utf-8')
 
-            if self.etag != '"%s"' % self.md5:
-                raise provider.storage_data_error(
-                    'ETag from S3 did not match computed MD5')
+            # If you use customer-provided encryption keys, the ETag value that
+            # Amazon S3 returns in the response will not be the MD5 of the
+            # object.
+            server_side_encryption_customer_algorithm = response.getheader(
+                'x-amz-server-side-encryption-customer-algorithm', None)
+            if server_side_encryption_customer_algorithm is None:
+                if self.etag != '"%s"' % md5:
+                    raise provider.storage_data_error(
+                        'ETag from S3 did not match computed MD5. '
+                        '%s vs. %s' % (self.etag, self.md5))
 
             return True
 
@@ -1053,7 +1107,7 @@ class Key(object):
             if provider.storage_class_header:
                 headers[provider.storage_class_header] = self.storage_class
 
-        if self.bucket != None:
+        if self.bucket is not None:
             if not replace:
                 if self.bucket.lookup(self.name):
                     return
@@ -1187,7 +1241,7 @@ class Key(object):
                 # What if different providers provide different classes?
         if hasattr(fp, 'name'):
             self.path = fp.name
-        if self.bucket != None:
+        if self.bucket is not None:
             if not md5 and provider.supports_chunked_transfer():
                 # defer md5 calculation to on the fly and
                 # we don't know anything about size yet.
@@ -1226,7 +1280,7 @@ class Key(object):
                 self.md5 = md5[0]
                 self.base64md5 = md5[1]
 
-            if self.name == None:
+            if self.name is None:
                 self.name = self.md5
             if not replace:
                 if self.bucket.lookup(self.name):
@@ -1305,7 +1359,7 @@ class Key(object):
                                                reduced_redundancy,
                                                encrypt_key=encrypt_key)
 
-    def set_contents_from_string(self, s, headers=None, replace=True,
+    def set_contents_from_string(self, string_data, headers=None, replace=True,
                                  cb=None, num_cb=10, policy=None, md5=None,
                                  reduced_redundancy=False,
                                  encrypt_key=False):
@@ -1362,9 +1416,9 @@ class Key(object):
             be encrypted on the server-side by S3 and will be stored
             in an encrypted form while at rest in S3.
         """
-        if isinstance(s, unicode):
-            s = s.encode("utf-8")
-        fp = StringIO.StringIO(s)
+        if not isinstance(string_data, bytes):
+            string_data = string_data.encode("utf-8")
+        fp = BytesIO(string_data)
         r = self.set_contents_from_file(fp, headers, replace, cb, num_cb,
                                         policy, md5, reduced_redundancy,
                                         encrypt_key=encrypt_key)
@@ -1409,6 +1463,14 @@ class Key(object):
             headers/values that will override any headers associated
             with the stored object in the response.  See
             http://goo.gl/EWOPb for details.
+
+        :type version_id: str
+        :param version_id: The ID of a particular version of the object.
+            If this parameter is not supplied but the Key object has
+            a ``version_id`` attribute, that value will be used when
+            retrieving the object.  You can set the Key object's
+            ``version_id`` attribute to None to always grab the latest
+            version from a version-enabled bucket.
         """
         self._get_file_internal(fp, headers=headers, cb=cb, num_cb=num_cb,
                                 torrent=torrent, version_id=version_id,
@@ -1444,7 +1506,7 @@ class Key(object):
         if response_headers:
             for key in response_headers:
                 query_args.append('%s=%s' % (
-                    key, urllib.quote(response_headers[key])))
+                    key, urllib.parse.quote(response_headers[key])))
         query_args = '&'.join(query_args)
         self.open('r', headers, query_args=query_args,
                   override_num_retries=override_num_retries)
@@ -1480,7 +1542,7 @@ class Key(object):
                     if i == cb_count or cb_count == -1:
                         cb(data_len, cb_size)
                         i = 0
-        except IOError, e:
+        except IOError as e:
             if e.errno == errno.ENOSPC:
                 raise StorageDataError('Out of space for destination file '
                                        '%s' % fp.name)
@@ -1566,8 +1628,16 @@ class Key(object):
             headers/values that will override any headers associated
             with the stored object in the response.  See
             http://goo.gl/EWOPb for details.
+
+        :type version_id: str
+        :param version_id: The ID of a particular version of the object.
+            If this parameter is not supplied but the Key object has
+            a ``version_id`` attribute, that value will be used when
+            retrieving the object.  You can set the Key object's
+            ``version_id`` attribute to None to always grab the latest
+            version from a version-enabled bucket.
         """
-        if self.bucket != None:
+        if self.bucket is not None:
             if res_download_handler:
                 res_download_handler.get_file(self, fp, headers, cb, num_cb,
                                               torrent=torrent,
@@ -1622,6 +1692,14 @@ class Key(object):
             headers/values that will override any headers associated
             with the stored object in the response.  See
             http://goo.gl/EWOPb for details.
+
+        :type version_id: str
+        :param version_id: The ID of a particular version of the object.
+            If this parameter is not supplied but the Key object has
+            a ``version_id`` attribute, that value will be used when
+            retrieving the object.  You can set the Key object's
+            ``version_id`` attribute to None to always grab the latest
+            version from a version-enabled bucket.
         """
         try:
             with open(filename, 'wb') as fp:
@@ -1634,10 +1712,10 @@ class Key(object):
             os.remove(filename)
             raise
         # if last_modified date was sent from s3, try to set file's timestamp
-        if self.last_modified != None:
+        if self.last_modified is not None:
             try:
-                modified_tuple = rfc822.parsedate_tz(self.last_modified)
-                modified_stamp = int(rfc822.mktime_tz(modified_tuple))
+                modified_tuple = email.utils.parsedate_tz(self.last_modified)
+                modified_stamp = int(email.utils.mktime_tz(modified_tuple))
                 os.utime(fp.name, (modified_stamp, modified_stamp))
             except Exception:
                 pass
@@ -1646,7 +1724,7 @@ class Key(object):
                                cb=None, num_cb=10,
                                torrent=False,
                                version_id=None,
-                               response_headers=None):
+                               response_headers=None, encoding=None):
         """
         Retrieve an object from S3 using the name of the Key object as the
         key in S3.  Return the contents of the object as a string.
@@ -1680,14 +1758,32 @@ class Key(object):
             with the stored object in the response.  See
             http://goo.gl/EWOPb for details.
 
-        :rtype: string
-        :returns: The contents of the file as a string
+        :type version_id: str
+        :param version_id: The ID of a particular version of the object.
+            If this parameter is not supplied but the Key object has
+            a ``version_id`` attribute, that value will be used when
+            retrieving the object.  You can set the Key object's
+            ``version_id`` attribute to None to always grab the latest
+            version from a version-enabled bucket.
+
+        :type encoding: str
+        :param encoding: The text encoding to use, such as ``utf-8``
+            or ``iso-8859-1``. If set, then a string will be returned.
+            Defaults to ``None`` and returns bytes.
+
+        :rtype: bytes or str
+        :returns: The contents of the file as bytes or a string
         """
-        fp = StringIO.StringIO()
+        fp = BytesIO()
         self.get_contents_to_file(fp, headers, cb, num_cb, torrent=torrent,
                                   version_id=version_id,
                                   response_headers=response_headers)
-        return fp.getvalue()
+        value = fp.getvalue()
+
+        if encoding is not None:
+            value = value.decode(encoding)
+
+        return value
 
     def add_email_grant(self, permission, email_address, headers=None):
         """
