@@ -45,7 +45,7 @@ class SQSConnection(AWSQueryConnection):
                  is_secure=True, port=None, proxy=None, proxy_port=None,
                  proxy_user=None, proxy_pass=None, debug=0,
                  https_connection_factory=None, region=None, path='/',
-                 security_token=None, validate_certs=True):
+                 security_token=None, validate_certs=True, profile_name=None):
         if not region:
             region = SQSRegionInfo(self, self.DefaultRegionName,
                                    self.DefaultRegionEndpoint)
@@ -58,7 +58,8 @@ class SQSConnection(AWSQueryConnection):
                                     self.region.endpoint, debug,
                                     https_connection_factory, path,
                                     security_token=security_token,
-                                    validate_certs=validate_certs)
+                                    validate_certs=validate_certs,
+                                    profile_name=profile_name)
         self.auth_region_name = self.region.name
 
     def _required_auth_capability(self):
@@ -122,12 +123,18 @@ class SQSConnection(AWSQueryConnection):
             supplied, the default is to return all attributes.  Valid
             attributes are:
 
+            * All
             * ApproximateNumberOfMessages
             * ApproximateNumberOfMessagesNotVisible
             * VisibilityTimeout
             * CreatedTimestamp
             * LastModifiedTimestamp
             * Policy
+            * MaximumMessageSize
+            * MessageRetentionPeriod
+            * QueueArn
+            * ApproximateNumberOfMessagesDelayed
+            * DelaySeconds
             * ReceiveMessageWaitTimeSeconds
             * RedrivePolicy
 
@@ -144,7 +151,7 @@ class SQSConnection(AWSQueryConnection):
 
     def receive_message(self, queue, number_messages=1,
                         visibility_timeout=None, attributes=None,
-                        wait_time_seconds=None):
+                        wait_time_seconds=None, message_attributes=None):
         """
         Read messages from an SQS Queue.
 
@@ -177,6 +184,11 @@ class SQSConnection(AWSQueryConnection):
             If a message is available, the call will return sooner than
             wait_time_seconds.
 
+        :type message_attributes: list
+        :param message_attributes: The name(s) of additional message
+            attributes to return. The default is to return no additional
+            message attributes. Use ``['All']`` or ``['.*']`` to return all.
+
         :rtype: list
         :return: A list of :class:`boto.sqs.message.Message` objects.
 
@@ -188,6 +200,9 @@ class SQSConnection(AWSQueryConnection):
             self.build_list_params(params, attributes, 'AttributeName')
         if wait_time_seconds is not None:
             params['WaitTimeSeconds'] = wait_time_seconds
+        if message_attributes is not None:
+            self.build_list_params(params, message_attributes,
+                                   'MessageAttributeName')
         return self.get_list('ReceiveMessage', params,
                              [('Message', queue.message_class)],
                              queue.id, queue)
@@ -244,10 +259,62 @@ class SQSConnection(AWSQueryConnection):
         params = {'ReceiptHandle' : receipt_handle}
         return self.get_status('DeleteMessage', params, queue.id)
 
-    def send_message(self, queue, message_content, delay_seconds=None):
+    def send_message(self, queue, message_content, delay_seconds=None,
+                     message_attributes=None):
+        """
+        Send a new message to the queue.
+
+        :type queue: A :class:`boto.sqs.queue.Queue` object.
+        :param queue: The Queue to which the messages will be written.
+
+        :type message_content: string
+        :param message_content: The body of the message
+
+        :type delay_seconds: int
+        :param delay_seconds: Number of seconds (0 - 900) to delay this
+            message from being processed.
+
+        :type message_attributes: dict
+        :param message_attributes: Message attributes to set. Should be
+            of the form:
+
+            {
+                "name1": {
+                    "data_type": "Number",
+                    "string_value": "1"
+                },
+                "name2": {
+                    "data_type": "String",
+                    "string_value": "Bob"
+                }
+            }
+
+        """
         params = {'MessageBody' : message_content}
         if delay_seconds:
             params['DelaySeconds'] = int(delay_seconds)
+
+        if message_attributes is not None:
+            keys = sorted(message_attributes.keys())
+            for i, name in enumerate(keys, start=1):
+                attribute = message_attributes[name]
+                params['MessageAttribute.%s.Name' % i] = name
+                if 'data_type' in attribute:
+                    params['MessageAttribute.%s.Value.DataType' % i] = \
+                        attribute['data_type']
+                if 'string_value' in attribute:
+                    params['MessageAttribute.%s.Value.StringValue' % i] = \
+                        attribute['string_value']
+                if 'binary_value' in attribute:
+                    params['MessageAttribute.%s.Value.BinaryValue' % i] = \
+                        attribute['binary_value']
+                if 'string_list_value' in attribute:
+                    params['MessageAttribute.%s.Value.StringListValue' % i] = \
+                        attribute['string_list_value']
+                if 'binary_list_value' in attribute:
+                    params['MessageAttribute.%s.Value.BinaryListValue' % i] = \
+                        attribute['binary_list_value']
+
         return self.get_object('SendMessage', params, Message,
                                queue.id, verb='POST')
 
@@ -263,19 +330,45 @@ class SQSConnection(AWSQueryConnection):
             tuple represents a single message to be written
             and consists of and ID (string) that must be unique
             within the list of messages, the message body itself
-            which can be a maximum of 64K in length, and an
+            which can be a maximum of 64K in length, an
             integer which represents the delay time (in seconds)
             for the message (0-900) before the message will
-            be delivered to the queue.
+            be delivered to the queue, and an optional dict of
+            message attributes like those passed to ``send_message``
+            above.
+
         """
         params = {}
         for i, msg in enumerate(messages):
-            p_name = 'SendMessageBatchRequestEntry.%i.Id' % (i+1)
-            params[p_name] = msg[0]
-            p_name = 'SendMessageBatchRequestEntry.%i.MessageBody' % (i+1)
-            params[p_name] = msg[1]
-            p_name = 'SendMessageBatchRequestEntry.%i.DelaySeconds' % (i+1)
-            params[p_name] = msg[2]
+            base = 'SendMessageBatchRequestEntry.%i' % (i + 1)
+            params['%s.Id' % base] = msg[0]
+            params['%s.MessageBody' % base] = msg[1]
+            params['%s.DelaySeconds' % base] = msg[2]
+            if len(msg) > 3:
+                base += '.MessageAttribute'
+                keys = sorted(msg[3].keys())
+                for j, name in enumerate(keys):
+                    attribute = msg[3][name]
+
+                    p_name = '%s.%i.Name' % (base, j + 1)
+                    params[p_name] = name
+
+                    if 'data_type' in attribute:
+                        p_name = '%s.%i.DataType' % (base, j + 1)
+                        params[p_name] = attribute['data_type']
+                    if 'string_value' in attribute:
+                        p_name = '%s.%i.StringValue' % (base, j + 1)
+                        params[p_name] = attribute['string_value']
+                    if 'binary_value' in attribute:
+                        p_name = '%s.%i.BinaryValue' % (base, j + 1)
+                        params[p_name] = attribute['binary_value']
+                    if 'string_list_value' in attribute:
+                        p_name = '%s.%i.StringListValue' % (base, j + 1)
+                        params[p_name] = attribute['string_list_value']
+                    if 'binary_list_value' in attribute:
+                        p_name = '%s.%i.BinaryListValue' % (base, j + 1)
+                        params[p_name] = attribute['binary_list_value']
+
         return self.get_object('SendMessageBatch', params, BatchResults,
                                queue.id, verb='POST')
 
