@@ -24,8 +24,6 @@
 """
 Some unit tests for the SQSConnection
 """
-from __future__ import with_statement
-
 import time
 from threading import Timer
 from tests.unit import unittest
@@ -41,7 +39,7 @@ class SQSConnectionTest(unittest.TestCase):
     sqs = True
 
     def test_1_basic(self):
-        print '--- running SQSConnection tests ---'
+        print('--- running SQSConnection tests ---')
         c = SQSConnection()
         rs = c.get_all_queues()
         num_queues = 0
@@ -152,7 +150,7 @@ class SQSConnectionTest(unittest.TestCase):
         m = queue_2.read()
         assert m['foo'] == 'bar'
 
-        print '--- tests completed ---'
+        print('--- tests completed ---')
 
     def test_sqs_timeout(self):
         c = SQSConnection()
@@ -213,9 +211,9 @@ class SQSConnectionTest(unittest.TestCase):
         self.assertEqual(response.id, messages[0].id)
         self.assertEqual(response.get_body(), messages[0].get_body())
         # The timer thread should send the message in 5 seconds, so
-        # we're giving +- .5 seconds for the total time the queue
+        # we're giving +- 1 second for the total time the queue
         # was blocked on the read call.
-        self.assertTrue(4.5 <= (end - start) <= 5.5)
+        self.assertTrue(4.0 <= (end - start) <= 6.0)
 
     def test_queue_deletion_affects_full_queues(self):
         conn = SQSConnection()
