@@ -1,5 +1,4 @@
-import mock
-import unittest
+from tests.compat import mock, unittest
 from boto.dynamodb2 import exceptions
 from boto.dynamodb2.fields import (HashKey, RangeKey,
                                    AllIndex, KeysOnlyIndex, IncludeIndex,
@@ -12,6 +11,7 @@ from boto.dynamodb2.table import Table
 from boto.dynamodb2.types import (STRING, NUMBER, BINARY,
                                   FILTER_OPERATORS, QUERY_OPERATORS)
 from boto.exception import JSONResponseError
+from boto.compat import six, long_type
 
 
 FakeDynamoDBConnection = mock.create_autospec(DynamoDBConnection)
@@ -337,6 +337,9 @@ class IndexFieldTestCase(unittest.TestCase):
 
 
 class ItemTestCase(unittest.TestCase):
+    if six.PY2:
+        assertCountEqual = unittest.TestCase.assertItemsEqual
+
     def setUp(self):
         super(ItemTestCase, self).setUp()
         self.table = Table('whatever', connection=FakeDynamoDBConnection())
@@ -368,31 +371,25 @@ class ItemTestCase(unittest.TestCase):
     # ordering everywhere & no erroneous failures.
 
     def test_keys(self):
-        self.assertEqual(sorted(self.johndoe.keys()), [
+        self.assertCountEqual(self.johndoe.keys(), [
             'date_joined',
             'first_name',
             'username',
         ])
 
     def test_values(self):
-        self.assertEqual(sorted(self.johndoe.values()), [
-            12345,
-            'John',
-            'johndoe',
-        ])
+        self.assertCountEqual(self.johndoe.values(),
+                              [12345, 'John', 'johndoe'])
 
     def test_contains(self):
-        self.assertTrue('username' in self.johndoe)
-        self.assertTrue('first_name' in self.johndoe)
-        self.assertTrue('date_joined' in self.johndoe)
-        self.assertFalse('whatever' in self.johndoe)
+        self.assertIn('username', self.johndoe)
+        self.assertIn('first_name', self.johndoe)
+        self.assertIn('date_joined', self.johndoe)
+        self.assertNotIn('whatever', self.johndoe)
 
     def test_iter(self):
-        self.assertEqual(list(self.johndoe), [
-            'johndoe',
-            'John',
-            12345,
-        ])
+        self.assertCountEqual(self.johndoe,
+                              ['johndoe', 'John', 12345])
 
     def test_get(self):
         self.assertEqual(self.johndoe.get('username'), 'johndoe')
@@ -405,11 +402,13 @@ class ItemTestCase(unittest.TestCase):
         self.assertEqual(self.johndoe.get('last_name', True), True)
 
     def test_items(self):
-        self.assertEqual(sorted(self.johndoe.items()), [
-            ('date_joined', 12345),
-            ('first_name', 'John'),
-            ('username', 'johndoe'),
-        ])
+        self.assertCountEqual(
+            self.johndoe.items(),
+            [
+                ('date_joined', 12345),
+                ('first_name', 'John'),
+                ('username', 'johndoe'),
+            ])
 
     def test_attribute_access(self):
         self.assertEqual(self.johndoe['username'], 'johndoe')
@@ -598,12 +597,12 @@ class ItemTestCase(unittest.TestCase):
         })
 
         self.johndoe['friends'] = set(['jane', 'alice'])
-        self.assertEqual(self.johndoe.prepare_full(), {
-            'username': {'S': 'johndoe'},
-            'first_name': {'S': 'John'},
-            'date_joined': {'N': '12345'},
-            'friends': {'SS': ['jane', 'alice']},
-        })
+        data = self.johndoe.prepare_full()
+        self.assertEqual(data['username'], {'S': 'johndoe'})
+        self.assertEqual(data['first_name'], {'S': 'John'})
+        self.assertEqual(data['date_joined'], {'N': '12345'})
+        self.assertCountEqual(data['friends']['SS'],
+                              ['jane', 'alice'])
 
     def test_prepare_full_empty_set(self):
         self.johndoe['friends'] = set()
@@ -934,55 +933,55 @@ class ResultSetTestCase(unittest.TestCase):
 
     def test_iteration(self):
         # First page.
-        self.assertEqual(self.results.next(), 'Hello john #0')
-        self.assertEqual(self.results.next(), 'Hello john #1')
-        self.assertEqual(self.results.next(), 'Hello john #2')
-        self.assertEqual(self.results.next(), 'Hello john #3')
-        self.assertEqual(self.results.next(), 'Hello john #4')
+        self.assertEqual(next(self.results), 'Hello john #0')
+        self.assertEqual(next(self.results), 'Hello john #1')
+        self.assertEqual(next(self.results), 'Hello john #2')
+        self.assertEqual(next(self.results), 'Hello john #3')
+        self.assertEqual(next(self.results), 'Hello john #4')
         self.assertEqual(self.results._limit, 15)
         # Second page.
-        self.assertEqual(self.results.next(), 'Hello john #5')
-        self.assertEqual(self.results.next(), 'Hello john #6')
-        self.assertEqual(self.results.next(), 'Hello john #7')
-        self.assertEqual(self.results.next(), 'Hello john #8')
-        self.assertEqual(self.results.next(), 'Hello john #9')
+        self.assertEqual(next(self.results), 'Hello john #5')
+        self.assertEqual(next(self.results), 'Hello john #6')
+        self.assertEqual(next(self.results), 'Hello john #7')
+        self.assertEqual(next(self.results), 'Hello john #8')
+        self.assertEqual(next(self.results), 'Hello john #9')
         self.assertEqual(self.results._limit, 10)
         # Third page.
-        self.assertEqual(self.results.next(), 'Hello john #10')
-        self.assertEqual(self.results.next(), 'Hello john #11')
-        self.assertEqual(self.results.next(), 'Hello john #12')
+        self.assertEqual(next(self.results), 'Hello john #10')
+        self.assertEqual(next(self.results), 'Hello john #11')
+        self.assertEqual(next(self.results), 'Hello john #12')
         self.assertRaises(StopIteration, self.results.next)
         self.assertEqual(self.results._limit, 7)
 
     def test_limit_smaller_than_first_page(self):
         results = ResultSet()
         results.to_call(fake_results, 'john', greeting='Hello', limit=2)
-        self.assertEqual(results.next(), 'Hello john #0')
-        self.assertEqual(results.next(), 'Hello john #1')
+        self.assertEqual(next(results), 'Hello john #0')
+        self.assertEqual(next(results), 'Hello john #1')
         self.assertRaises(StopIteration, results.next)
 
     def test_limit_equals_page(self):
         results = ResultSet()
         results.to_call(fake_results, 'john', greeting='Hello', limit=5)
         # First page
-        self.assertEqual(results.next(), 'Hello john #0')
-        self.assertEqual(results.next(), 'Hello john #1')
-        self.assertEqual(results.next(), 'Hello john #2')
-        self.assertEqual(results.next(), 'Hello john #3')
-        self.assertEqual(results.next(), 'Hello john #4')
+        self.assertEqual(next(results), 'Hello john #0')
+        self.assertEqual(next(results), 'Hello john #1')
+        self.assertEqual(next(results), 'Hello john #2')
+        self.assertEqual(next(results), 'Hello john #3')
+        self.assertEqual(next(results), 'Hello john #4')
         self.assertRaises(StopIteration, results.next)
 
     def test_limit_greater_than_page(self):
         results = ResultSet()
         results.to_call(fake_results, 'john', greeting='Hello', limit=6)
         # First page
-        self.assertEqual(results.next(), 'Hello john #0')
-        self.assertEqual(results.next(), 'Hello john #1')
-        self.assertEqual(results.next(), 'Hello john #2')
-        self.assertEqual(results.next(), 'Hello john #3')
-        self.assertEqual(results.next(), 'Hello john #4')
+        self.assertEqual(next(results), 'Hello john #0')
+        self.assertEqual(next(results), 'Hello john #1')
+        self.assertEqual(next(results), 'Hello john #2')
+        self.assertEqual(next(results), 'Hello john #3')
+        self.assertEqual(next(results), 'Hello john #4')
         # Second page
-        self.assertEqual(results.next(), 'Hello john #5')
+        self.assertEqual(next(results), 'Hello john #5')
         self.assertRaises(StopIteration, results.next)
 
     def test_iteration_noresults(self):
@@ -1038,15 +1037,15 @@ class ResultSetTestCase(unittest.TestCase):
         results = ResultSet()
         results.to_call(sporadic(), limit=20)
         # First page
-        self.assertEqual(results.next(), 'Result #0')
-        self.assertEqual(results.next(), 'Result #1')
-        self.assertEqual(results.next(), 'Result #2')
-        self.assertEqual(results.next(), 'Result #3')
+        self.assertEqual(next(results), 'Result #0')
+        self.assertEqual(next(results), 'Result #1')
+        self.assertEqual(next(results), 'Result #2')
+        self.assertEqual(next(results), 'Result #3')
         # Second page (misses!)
         # Moves on to the third page
-        self.assertEqual(results.next(), 'Result #4')
-        self.assertEqual(results.next(), 'Result #5')
-        self.assertEqual(results.next(), 'Result #6')
+        self.assertEqual(next(results), 'Result #4')
+        self.assertEqual(next(results), 'Result #5')
+        self.assertEqual(next(results), 'Result #6')
         self.assertRaises(StopIteration, results.next)
 
     def test_list(self):
@@ -1127,10 +1126,10 @@ class BatchGetResultSetTestCase(unittest.TestCase):
 
     def test_iteration(self):
         # First page.
-        self.assertEqual(self.results.next(), 'hello alice')
-        self.assertEqual(self.results.next(), 'hello bob')
-        self.assertEqual(self.results.next(), 'hello jane')
-        self.assertEqual(self.results.next(), 'hello johndoe')
+        self.assertEqual(next(self.results), 'hello alice')
+        self.assertEqual(next(self.results), 'hello bob')
+        self.assertEqual(next(self.results), 'hello jane')
+        self.assertEqual(next(self.results), 'hello johndoe')
         self.assertRaises(StopIteration, self.results.next)
 
 
@@ -1550,33 +1549,36 @@ class TableTestCase(unittest.TestCase):
             self.assertEqual(self.users.throughput['read'], 9)
             self.assertEqual(self.users.throughput['write'], 5)
 
-        mock_update.assert_called_once_with(
-            'users',
-            global_secondary_index_updates=[
-                {
-                    'Update': {
-                        'IndexName': 'AnotherIndex',
-                        'ProvisionedThroughput': {
-                            'WriteCapacityUnits': 2,
-                            'ReadCapacityUnits': 1
-                        }
-                    }
-                },
-                {
-                    'Update': {
-                        'IndexName': 'WhateverIndex',
-                        'ProvisionedThroughput': {
-                            'WriteCapacityUnits': 1,
-                            'ReadCapacityUnits': 6
-                        }
+        args, kwargs = mock_update.call_args
+        self.assertEqual(args, ('users',))
+        self.assertEqual(kwargs['provisioned_throughput'], {
+            'WriteCapacityUnits': 5,
+            'ReadCapacityUnits': 9,
+            })
+        update = kwargs['global_secondary_index_updates'][:]
+        update.sort(key=lambda x: x['Update']['IndexName'])
+        self.assertDictEqual(
+            update[0],
+            {
+                'Update': {
+                    'IndexName': 'AnotherIndex',
+                    'ProvisionedThroughput': {
+                        'WriteCapacityUnits': 2,
+                        'ReadCapacityUnits': 1
                     }
                 }
-            ],
-            provisioned_throughput={
-                'WriteCapacityUnits': 5,
-                'ReadCapacityUnits': 9,
-            }
-        )
+            })
+        self.assertDictEqual(
+            update[1],
+            {
+                'Update': {
+                    'IndexName': 'WhateverIndex',
+                    'ProvisionedThroughput': {
+                        'WriteCapacityUnits': 1,
+                        'ReadCapacityUnits': 6
+                    }
+                }
+            })
 
     def test_delete(self):
         with mock.patch.object(
@@ -1774,7 +1776,40 @@ class TableTestCase(unittest.TestCase):
             'date_joined': {
                 'N': '23456'
             }
-        })
+        }, expected=None, conditional_operator=None)
+
+    def test_delete_item_conditionally(self):
+        with mock.patch.object(
+                self.users.connection,
+                'delete_item',
+                return_value={}) as mock_delete_item:
+            self.assertTrue(self.users.delete_item(expected={'balance__eq': 0},
+                                                   username='johndoe', date_joined=23456))
+
+        mock_delete_item.assert_called_once_with('users', {
+            'username': {
+                'S': 'johndoe'
+            },
+            'date_joined': {
+                'N': '23456'
+            }
+        },
+        expected={
+            'balance': {
+                'ComparisonOperator': 'EQ', 'AttributeValueList': [{'N': '0'}]
+                },
+            },
+        conditional_operator=None)
+
+        def side_effect(*args, **kwargs):
+            raise exceptions.ConditionalCheckFailedException(400, '', {})
+
+        with mock.patch.object(
+                self.users.connection,
+                'delete_item',
+                side_effect=side_effect) as mock_delete_item:
+            self.assertFalse(self.users.delete_item(expected={'balance__eq': 0},
+                                                    username='johndoe', date_joined=23456))
 
     def test_get_key_fields_no_schema_populated(self):
         expected = {
@@ -1898,7 +1933,7 @@ class TableTestCase(unittest.TestCase):
             try:
                 with self.users.batch_write() as batch:
                     raise Exception('OH NOES')
-            except Exception, e:
+            except Exception as e:
                 self.assertEqual(str(e), 'OH NOES')
 
         self.assertFalse(mock_batch.called)
@@ -2393,11 +2428,11 @@ class TableTestCase(unittest.TestCase):
                 results,
                 'the_callable',
                 return_value=items_1) as mock_query:
-            res_1 = results.next()
+            res_1 = next(results)
             # Now it should be populated.
             self.assertEqual(len(results._results), 2)
             self.assertEqual(res_1['username'], 'johndoe')
-            res_2 = results.next()
+            res_2 = next(results)
             self.assertEqual(res_2['username'], 'jane')
 
         self.assertEqual(mock_query.call_count, 1)
@@ -2416,7 +2451,7 @@ class TableTestCase(unittest.TestCase):
                 results,
                 'the_callable',
                 return_value=items_2) as mock_query_2:
-            res_3 = results.next()
+            res_3 = next(results)
             # New results should have been found.
             self.assertEqual(len(results._results), 1)
             self.assertEqual(res_3['username'], 'foodoe')
@@ -2448,12 +2483,12 @@ class TableTestCase(unittest.TestCase):
                 results,
                 'the_callable',
                 return_value=items_1) as mock_query:
-            res_1 = results.next()
+            res_1 = next(results)
             # Now it should be populated.
             self.assertEqual(len(results._results), 2)
             self.assertEqual(res_1['username'], 'johndoe')
-            self.assertEqual(res_1.keys(), ['username'])
-            res_2 = results.next()
+            self.assertEqual(list(res_1.keys()), ['username'])
+            res_2 = next(results)
             self.assertEqual(res_2['username'], 'jane')
 
         self.assertEqual(mock_query.call_count, 1)
@@ -2484,11 +2519,11 @@ class TableTestCase(unittest.TestCase):
                 results,
                 'the_callable',
                 return_value=items_1) as mock_scan:
-            res_1 = results.next()
+            res_1 = next(results)
             # Now it should be populated.
             self.assertEqual(len(results._results), 2)
             self.assertEqual(res_1['username'], 'johndoe')
-            res_2 = results.next()
+            res_2 = next(results)
             self.assertEqual(res_2['username'], 'jane')
 
         self.assertEqual(mock_scan.call_count, 1)
@@ -2507,7 +2542,7 @@ class TableTestCase(unittest.TestCase):
                 results,
                 'the_callable',
                 return_value=items_2) as mock_scan_2:
-            res_3 = results.next()
+            res_3 = next(results)
             # New results should have been found.
             self.assertEqual(len(results._results), 1)
             self.assertEqual(res_3['username'], 'zoeydoe')
@@ -2538,12 +2573,12 @@ class TableTestCase(unittest.TestCase):
                 results,
                 'the_callable',
                 return_value=items_1) as mock_query:
-            res_1 = results.next()
+            res_1 = next(results)
             # Now it should be populated.
             self.assertEqual(len(results._results), 2)
             self.assertEqual(res_1['username'], 'johndoe')
-            self.assertEqual(res_1.keys(), ['username'])
-            res_2 = results.next()
+            self.assertEqual(list(res_1.keys()), ['username'])
+            res_2 = next(results)
             self.assertEqual(res_2['username'], 'jane')
 
         self.assertEqual(mock_query.call_count, 1)
@@ -2593,6 +2628,41 @@ class TableTestCase(unittest.TestCase):
                 return_value=expected) as mock_count:
             self.assertEqual(self.users.count(), 5)
 
+    def test_query_count_simple(self):
+        expected_0 = {
+            'Count': 0.0,
+        }
+
+        expected_1 = {
+            'Count': 10.0,
+        }
+
+        with mock.patch.object(
+                self.users.connection,
+                'query',
+                return_value=expected_0) as mock_query:
+            results = self.users.query_count(username__eq='notmyname')
+            self.assertTrue(isinstance(results, int))
+            self.assertEqual(results, 0)
+        self.assertEqual(mock_query.call_count, 1)
+        self.assertIn('scan_index_forward', mock_query.call_args[1])
+        self.assertEqual(True, mock_query.call_args[1]['scan_index_forward'])
+        self.assertIn('limit', mock_query.call_args[1])
+        self.assertEqual(None, mock_query.call_args[1]['limit'])
+
+        with mock.patch.object(
+                self.users.connection,
+                'query',
+                return_value=expected_1) as mock_query:
+            results = self.users.query_count(username__gt='somename', consistent=True, scan_index_forward=False, limit=10)
+            self.assertTrue(isinstance(results, int))
+            self.assertEqual(results, 10)
+        self.assertEqual(mock_query.call_count, 1)
+        self.assertIn('scan_index_forward', mock_query.call_args[1])
+        self.assertEqual(False, mock_query.call_args[1]['scan_index_forward'])
+        self.assertIn('limit', mock_query.call_args[1])
+        self.assertEqual(10, mock_query.call_args[1]['limit'])
+
     def test_private_batch_get(self):
         expected = {
             "ConsumedCapacity": {
@@ -2705,6 +2775,52 @@ class TableTestCase(unittest.TestCase):
             }
         })
 
+    def test_private_batch_get_attributes(self):
+        # test if AttributesToGet parameter is passed to DynamoDB API
+        expected = {
+            "ConsumedCapacity": {
+                "CapacityUnits": 0.5,
+                "TableName": "users"
+            },
+            'Responses': {
+                'users': [
+                    {
+                        'username': {'S': 'alice'},
+                        'first_name': {'S': 'Alice'},
+                    },
+                    {
+                        'username': {'S': 'bob'},
+                        'first_name': {'S': 'Bob'},
+                    },
+                ],
+            },
+            "UnprocessedKeys": {},
+        }
+
+        with mock.patch.object(
+                self.users.connection,
+                'batch_get_item',
+                return_value=expected) as mock_batch_get_attr:
+            results = self.users._batch_get(keys=[
+                    {'username': 'alice'},
+                    {'username': 'bob'},
+                ], attributes=['username', 'first_name'])
+            usernames = [res['username'] for res in results['results']]
+            first_names = [res['first_name'] for res in results['results']]
+            self.assertEqual(usernames, ['alice', 'bob'])
+            self.assertEqual(first_names, ['Alice', 'Bob'])
+            self.assertEqual(len(results['results']), 2)
+            self.assertEqual(results['last_key'], None)
+            self.assertEqual(results['unprocessed_keys'], [])
+
+        mock_batch_get_attr.assert_called_once_with(request_items={
+            'users': {
+                'Keys': [ { 'username': {'S': 'alice'} },
+                          { 'username': {'S': 'bob'} }, ],
+                'AttributesToGet': ['username', 'first_name'],
+            },
+        })
+
     def test_batch_get(self):
         items_1 = {
             'results': [
@@ -2738,11 +2854,11 @@ class TableTestCase(unittest.TestCase):
                 results,
                 'the_callable',
                 return_value=items_1) as mock_batch_get:
-            res_1 = results.next()
+            res_1 = next(results)
             # Now it should be populated.
             self.assertEqual(len(results._results), 2)
             self.assertEqual(res_1['username'], 'johndoe')
-            res_2 = results.next()
+            res_2 = next(results)
             self.assertEqual(res_2['username'], 'jane')
 
         self.assertEqual(mock_batch_get.call_count, 1)
@@ -2762,7 +2878,7 @@ class TableTestCase(unittest.TestCase):
                 results,
                 'the_callable',
                 return_value=items_2) as mock_batch_get_2:
-            res_3 = results.next()
+            res_3 = next(results)
             # New results should have been found.
             self.assertEqual(len(results._results), 1)
             self.assertEqual(res_3['username'], 'zoeydoe')
