@@ -26,7 +26,7 @@ class TestEC2ConnectionBase(AWSMockServiceTestCase):
 class TestReservedInstanceOfferings(TestEC2ConnectionBase):
 
     def default_body(self):
-        return """
+        return b"""
             <DescribeReservedInstancesOfferingsResponse>
                 <requestId>d3253568-edcf-4897-9a3d-fb28e0b3fa38</requestId>
                     <reservedInstancesOfferingsSet>
@@ -105,7 +105,7 @@ class TestReservedInstanceOfferings(TestEC2ConnectionBase):
     def test_get_reserved_instance_offerings_params(self):
         self.set_http_response(status_code=200)
         self.ec2.get_all_reserved_instances_offerings(
-            reserved_instances_offering_ids=['id1','id2'],
+            reserved_instances_offering_ids=['id1', 'id2'],
             instance_type='t1.micro',
             availability_zone='us-east-1',
             product_description='description',
@@ -132,33 +132,33 @@ class TestReservedInstanceOfferings(TestEC2ConnectionBase):
             'MaxDuration': '1000',
             'MaxInstanceCount': '1',
             'NextToken': 'next_token',
-            'MaxResults': '10',},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp', 'Version'])
+            'MaxResults': '10', },
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp', 'Version'])
 
 
 class TestPurchaseReservedInstanceOffering(TestEC2ConnectionBase):
     def default_body(self):
-        return """<PurchaseReservedInstancesOffering />"""
+        return b"""<PurchaseReservedInstancesOffering />"""
 
     def test_serialized_api_args(self):
         self.set_http_response(status_code=200)
         response = self.ec2.purchase_reserved_instance_offering(
-                'offering_id', 1, (100.0, 'USD'))
+            'offering_id', 1, (100.0, 'USD'))
         self.assert_request_parameters({
             'Action': 'PurchaseReservedInstancesOffering',
             'InstanceCount': 1,
             'ReservedInstancesOfferingId': 'offering_id',
             'LimitPrice.Amount': '100.0',
-            'LimitPrice.CurrencyCode': 'USD',},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp',
-                                   'Version'])
+            'LimitPrice.CurrencyCode': 'USD', },
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp',
+                                  'Version'])
 
 
 class TestCreateImage(TestEC2ConnectionBase):
     def default_body(self):
-        return """<CreateImageResponse xmlns="http://ec2.amazonaws.com/doc/2013-10-01/">
+        return b"""<CreateImageResponse xmlns="http://ec2.amazonaws.com/doc/2013-10-01/">
    <requestId>59dbff89-35bd-4eac-99ed-be587EXAMPLE</requestId>
    <imageId>ami-4fa54026</imageId>
 </CreateImageResponse>"""
@@ -166,35 +166,35 @@ class TestCreateImage(TestEC2ConnectionBase):
     def test_minimal(self):
         self.set_http_response(status_code=200)
         response = self.ec2.create_image(
-                'instance_id', 'name')
+            'instance_id', 'name')
         self.assert_request_parameters({
             'Action': 'CreateImage',
             'InstanceId': 'instance_id',
             'Name': 'name'},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp',
-                                   'Version'])
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp',
+                                  'Version'])
 
     def test_block_device_mapping(self):
         self.set_http_response(status_code=200)
         bdm = BlockDeviceMapping()
         bdm['test'] = BlockDeviceType()
         response = self.ec2.create_image(
-                'instance_id', 'name', block_device_mapping=bdm)
+            'instance_id', 'name', block_device_mapping=bdm)
         self.assert_request_parameters({
             'Action': 'CreateImage',
             'InstanceId': 'instance_id',
             'Name': 'name',
             'BlockDeviceMapping.1.DeviceName': 'test',
             'BlockDeviceMapping.1.Ebs.DeleteOnTermination': 'false'},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp',
-                                   'Version'])
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp',
+                                  'Version'])
 
 
 class TestCancelReservedInstancesListing(TestEC2ConnectionBase):
     def default_body(self):
-        return """
+        return b"""
             <CancelReservedInstancesListingResponse>
                 <requestId>request_id</requestId>
                 <reservedInstancesListingsSet>
@@ -283,7 +283,7 @@ class TestCancelReservedInstancesListing(TestEC2ConnectionBase):
 
 class TestCreateReservedInstancesListing(TestEC2ConnectionBase):
     def default_body(self):
-        return """
+        return b"""
             <CreateReservedInstancesListingResponse>
                 <requestId>request_id</requestId>
                 <reservedInstancesListingsSet>
@@ -414,15 +414,15 @@ class TestCreateReservedInstancesListing(TestEC2ConnectionBase):
             'PriceSchedules.0.Price': '2.5',
             'PriceSchedules.0.Term': '11',
             'PriceSchedules.1.Price': '2.0',
-            'PriceSchedules.1.Term': '8',},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp',
-                                   'Version'])
+            'PriceSchedules.1.Term': '8', },
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp',
+                                  'Version'])
 
 
 class TestDescribeSpotInstanceRequests(TestEC2ConnectionBase):
     def default_body(self):
-        return """
+        return b"""
         <DescribeSpotInstanceRequestsResponse>
             <requestId>requestid</requestId>
             <spotInstanceRequestSet>
@@ -499,7 +499,7 @@ class TestDescribeSpotInstanceRequests(TestEC2ConnectionBase):
 
 class TestCopySnapshot(TestEC2ConnectionBase):
     def default_body(self):
-        return """
+        return b"""
         <CopySnapshotResponse xmlns="http://ec2.amazonaws.com/doc/2012-12-01/">
             <requestId>request_id</requestId>
             <snapshotId>snap-copied-id</snapshotId>
@@ -517,13 +517,14 @@ class TestCopySnapshot(TestEC2ConnectionBase):
             'Description': 'description',
             'SourceRegion': 'us-west-2',
             'SourceSnapshotId': 'snap-id'},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp',
-                                   'Version'])
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp',
+                                  'Version'])
+
 
 class TestCopyImage(TestEC2ConnectionBase):
     def default_body(self):
-        return """
+        return b"""
         <CopyImageResponse xmlns="http://ec2.amazonaws.com/doc/2013-07-15/">
            <requestId>request_id</requestId>
            <imageId>ami-copied-id</imageId>
@@ -533,7 +534,7 @@ class TestCopyImage(TestEC2ConnectionBase):
     def test_copy_image(self):
         self.set_http_response(status_code=200)
         copied_ami = self.ec2.copy_image('us-west-2', 'ami-id',
-                                     'name', 'description', 'client-token')
+                                         'name', 'description', 'client-token')
         self.assertEqual(copied_ami.image_id, 'ami-copied-id')
 
         self.assert_request_parameters({
@@ -543,9 +544,10 @@ class TestCopyImage(TestEC2ConnectionBase):
             'SourceRegion': 'us-west-2',
             'SourceImageId': 'ami-id',
             'ClientToken': 'client-token'},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp',
-                                   'Version'])
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp',
+                                  'Version'])
+
     def test_copy_image_without_name(self):
         self.set_http_response(status_code=200)
         copied_ami = self.ec2.copy_image('us-west-2', 'ami-id',
@@ -559,13 +561,14 @@ class TestCopyImage(TestEC2ConnectionBase):
             'SourceRegion': 'us-west-2',
             'SourceImageId': 'ami-id',
             'ClientToken': 'client-token'},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp',
-                                   'Version'])
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp',
+                                  'Version'])
+
 
 class TestAccountAttributes(TestEC2ConnectionBase):
     def default_body(self):
-        return """
+        return b"""
         <DescribeAccountAttributesResponse xmlns="http://ec2.amazonaws.com/doc/2012-12-01/">
             <requestId>6d042e8a-4bc3-43e8-8265-3cbc54753f14</requestId>
             <accountAttributeSet>
@@ -624,7 +627,7 @@ class TestAccountAttributes(TestEC2ConnectionBase):
 
 class TestDescribeVPCAttribute(TestEC2ConnectionBase):
     def default_body(self):
-        return """
+        return b"""
         <DescribeVpcAttributeResponse xmlns="http://ec2.amazonaws.com/doc/2013-02-01/">
             <requestId>request_id</requestId>
             <vpcId>vpc-id</vpcId>
@@ -643,15 +646,15 @@ class TestDescribeVPCAttribute(TestEC2ConnectionBase):
         self.assert_request_parameters({
             'Action': 'DescribeVpcAttribute',
             'VpcId': 'vpc-id',
-            'Attribute': 'enableDnsHostnames',},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp',
-                                   'Version'])
+            'Attribute': 'enableDnsHostnames', },
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp',
+                                  'Version'])
 
 
 class TestGetAllNetworkInterfaces(TestEC2ConnectionBase):
     def default_body(self):
-        return """
+        return b"""
 <DescribeNetworkInterfacesResponse xmlns="http://ec2.amazonaws.com/\
     doc/2013-06-15/">
     <requestId>fc45294c-006b-457b-bab9-012f5b3b0e40</requestId>
@@ -702,15 +705,28 @@ class TestGetAllNetworkInterfaces(TestEC2ConnectionBase):
     </networkInterfaceSet>
 </DescribeNetworkInterfacesResponse>"""
 
+    def test_get_all_network_interfaces(self):
+        self.set_http_response(status_code=200)
+        result = self.ec2.get_all_network_interfaces(network_interface_ids=['eni-0f62d866'])
+        self.assert_request_parameters({
+            'Action': 'DescribeNetworkInterfaces',
+            'NetworkInterfaceId.1': 'eni-0f62d866'},
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp',
+                                  'Version'])
+        self.assertEqual(len(result), 1)
+        self.assertEqual(result[0].id, 'eni-0f62d866')
+
     def test_attachment_has_device_index(self):
         self.set_http_response(status_code=200)
         parsed = self.ec2.get_all_network_interfaces()
 
         self.assertEqual(5, parsed[0].attachment.device_index)
 
+
 class TestGetAllImages(TestEC2ConnectionBase):
     def default_body(self):
-        return """
+        return b"""
 <DescribeImagesResponse xmlns="http://ec2.amazonaws.com/doc/2013-02-01/">
     <requestId>e32375e8-4ac3-4099-a8bf-3ec902b9023e</requestId>
     <imagesSet>
@@ -802,7 +818,7 @@ class TestGetAllImages(TestEC2ConnectionBase):
 
 class TestModifyInterfaceAttribute(TestEC2ConnectionBase):
     def default_body(self):
-        return """
+        return b"""
 <ModifyNetworkInterfaceAttributeResponse \
     xmlns="http://ec2.amazonaws.com/doc/2013-06-15/">
     <requestId>657a4623-5620-4232-b03b-427e852d71cf</requestId>
@@ -818,9 +834,9 @@ class TestModifyInterfaceAttribute(TestEC2ConnectionBase):
             'Action': 'ModifyNetworkInterfaceAttribute',
             'NetworkInterfaceId': 'id',
             'Description.Value': 'foo'},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp',
-                                   'Version'])
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp',
+                                  'Version'])
 
     def test_modify_source_dest_check_bool(self):
         self.set_http_response(status_code=200)
@@ -831,9 +847,9 @@ class TestModifyInterfaceAttribute(TestEC2ConnectionBase):
             'Action': 'ModifyNetworkInterfaceAttribute',
             'NetworkInterfaceId': 'id',
             'SourceDestCheck.Value': 'true'},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp',
-                                   'Version'])
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp',
+                                  'Version'])
 
     def test_modify_source_dest_check_str(self):
         self.set_http_response(status_code=200)
@@ -844,9 +860,9 @@ class TestModifyInterfaceAttribute(TestEC2ConnectionBase):
             'Action': 'ModifyNetworkInterfaceAttribute',
             'NetworkInterfaceId': 'id',
             'SourceDestCheck.Value': 'true'},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp',
-                                   'Version'])
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp',
+                                  'Version'])
 
     def test_modify_source_dest_check_invalid(self):
         self.set_http_response(status_code=200)
@@ -867,9 +883,9 @@ class TestModifyInterfaceAttribute(TestEC2ConnectionBase):
             'NetworkInterfaceId': 'id',
             'Attachment.AttachmentId': 'bar',
             'Attachment.DeleteOnTermination': 'true'},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp',
-                                   'Version'])
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp',
+                                  'Version'])
 
     def test_modify_delete_on_termination_bool(self):
         self.set_http_response(status_code=200)
@@ -883,9 +899,9 @@ class TestModifyInterfaceAttribute(TestEC2ConnectionBase):
             'NetworkInterfaceId': 'id',
             'Attachment.AttachmentId': 'bar',
             'Attachment.DeleteOnTermination': 'false'},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp',
-                                   'Version'])
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp',
+                                  'Version'])
 
     def test_modify_delete_on_termination_invalid(self):
         self.set_http_response(status_code=200)
@@ -906,9 +922,9 @@ class TestModifyInterfaceAttribute(TestEC2ConnectionBase):
             'NetworkInterfaceId': 'id',
             'SecurityGroupId.1': 'sg-1',
             'SecurityGroupId.2': 'sg-2'},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp',
-                                   'Version'])
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp',
+                                  'Version'])
 
     def test_modify_group_set_invalid(self):
         self.set_http_response(status_code=200)
@@ -932,7 +948,8 @@ class TestConnectToRegion(unittest.TestCase):
 
     def test_aws_region(self):
         region = boto.ec2.RegionData.keys()[0]
-        self.ec2 = boto.ec2.connect_to_region(region,
+        self.ec2 = boto.ec2.connect_to_region(
+            region,
             https_connection_factory=self.https_connection_factory,
             aws_access_key_id='aws_access_key_id',
             aws_secret_access_key='aws_secret_access_key'
@@ -940,16 +957,18 @@ class TestConnectToRegion(unittest.TestCase):
         self.assertEqual(boto.ec2.RegionData[region], self.ec2.host)
 
     def test_non_aws_region(self):
-        self.ec2 = boto.ec2.connect_to_region('foo',
+        self.ec2 = boto.ec2.connect_to_region(
+            'foo',
             https_connection_factory=self.https_connection_factory,
             aws_access_key_id='aws_access_key_id',
             aws_secret_access_key='aws_secret_access_key',
-            region = RegionInfo(name='foo', endpoint='https://foo.com/bar')
+            region=RegionInfo(name='foo', endpoint='https://foo.com/bar')
         )
         self.assertEqual('https://foo.com/bar', self.ec2.host)
 
     def test_missing_region(self):
-        self.ec2 = boto.ec2.connect_to_region('foo',
+        self.ec2 = boto.ec2.connect_to_region(
+            'foo',
             https_connection_factory=self.https_connection_factory,
             aws_access_key_id='aws_access_key_id',
             aws_secret_access_key='aws_secret_access_key'
@@ -1057,7 +1076,7 @@ class TestTrimSnapshots(TestEC2ConnectionBase):
 
 class TestModifyReservedInstances(TestEC2ConnectionBase):
     def default_body(self):
-        return """<ModifyReservedInstancesResponse xmlns='http://ec2.amazonaws.com/doc/2013-08-15/'>
+        return b"""<ModifyReservedInstancesResponse xmlns='http://ec2.amazonaws.com/doc/2013-08-15/'>
     <requestId>bef729b6-0731-4489-8881-2258746ae163</requestId>
     <reservedInstancesModificationId>rimod-3aae219d-3d63-47a9-a7e9-e764example</reservedInstancesModificationId>
 </ModifyReservedInstancesResponse>"""
@@ -1095,7 +1114,7 @@ class TestModifyReservedInstances(TestEC2ConnectionBase):
 
 class TestDescribeReservedInstancesModifications(TestEC2ConnectionBase):
     def default_body(self):
-        return """<DescribeReservedInstancesModificationsResponse xmlns='http://ec2.amazonaws.com/doc/2013-08-15/'>
+        return b"""<DescribeReservedInstancesModificationsResponse xmlns='http://ec2.amazonaws.com/doc/2013-08-15/'>
     <requestId>eb4a6e3c-3689-445c-b536-19e38df35898</requestId>
     <reservedInstancesModificationsSet>
         <item>
@@ -1195,7 +1214,7 @@ class TestDescribeReservedInstancesModifications(TestEC2ConnectionBase):
 
 class TestRegisterImage(TestEC2ConnectionBase):
     def default_body(self):
-        return """
+        return b"""
             <RegisterImageResponse xmlns="http://ec2.amazonaws.com/doc/2013-08-15/">
               <requestId>59dbff89-35bd-4eac-99ed-be587EXAMPLE</requestId>
               <imageId>ami-1a2b3c4d</imageId>
@@ -1253,38 +1272,37 @@ class TestRegisterImage(TestEC2ConnectionBase):
             'SignatureVersion', 'Timestamp',
             'Version'
         ])
-    
+
     def test_volume_delete_on_termination_on(self):
         self.set_http_response(status_code=200)
         self.ec2.register_image('name', 'description',
                                 snapshot_id='snap-12345678',
                                 delete_root_volume_on_termination=True)
-        
+
         self.assert_request_parameters({
             'Action': 'RegisterImage',
             'Name': 'name',
             'Description': 'description',
             'BlockDeviceMapping.1.DeviceName': None,
-            'BlockDeviceMapping.1.Ebs.DeleteOnTermination' : 'true',
+            'BlockDeviceMapping.1.Ebs.DeleteOnTermination': 'true',
             'BlockDeviceMapping.1.Ebs.SnapshotId': 'snap-12345678',
         }, ignore_params_values=[
             'AWSAccessKeyId', 'SignatureMethod',
             'SignatureVersion', 'Timestamp',
             'Version'
         ])
-        
 
     def test_volume_delete_on_termination_default(self):
         self.set_http_response(status_code=200)
         self.ec2.register_image('name', 'description',
                                 snapshot_id='snap-12345678')
-        
+
         self.assert_request_parameters({
             'Action': 'RegisterImage',
             'Name': 'name',
             'Description': 'description',
             'BlockDeviceMapping.1.DeviceName': None,
-            'BlockDeviceMapping.1.Ebs.DeleteOnTermination' : 'false',
+            'BlockDeviceMapping.1.Ebs.DeleteOnTermination': 'false',
             'BlockDeviceMapping.1.Ebs.SnapshotId': 'snap-12345678',
         }, ignore_params_values=[
             'AWSAccessKeyId', 'SignatureMethod',
@@ -1295,7 +1313,7 @@ class TestRegisterImage(TestEC2ConnectionBase):
 
 class TestTerminateInstances(TestEC2ConnectionBase):
     def default_body(self):
-        return """<?xml version="1.0" ?>
+        return b"""<?xml version="1.0" ?>
             <TerminateInstancesResponse xmlns="http://ec2.amazonaws.com/doc/2013-07-15/">
                 <requestId>req-59a9ad52-0434-470c-ad48-4f89ded3a03e</requestId>
                 <instancesSet>
@@ -1322,7 +1340,7 @@ class TestTerminateInstances(TestEC2ConnectionBase):
 class TestDescribeInstances(TestEC2ConnectionBase):
 
     def default_body(self):
-        return """
+        return b"""
             <DescribeInstancesResponse>
             </DescribeInstancesResponse>
         """
@@ -1332,20 +1350,20 @@ class TestDescribeInstances(TestEC2ConnectionBase):
         self.ec2.get_all_instances()
         self.assert_request_parameters({
             'Action': 'DescribeInstances'},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp', 'Version'])
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp', 'Version'])
 
         self.ec2.get_all_reservations()
         self.assert_request_parameters({
             'Action': 'DescribeInstances'},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp', 'Version'])
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp', 'Version'])
 
         self.ec2.get_only_instances()
         self.assert_request_parameters({
             'Action': 'DescribeInstances'},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp', 'Version'])
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp', 'Version'])
 
     def test_max_results(self):
         self.set_http_response(status_code=200)
@@ -1355,8 +1373,8 @@ class TestDescribeInstances(TestEC2ConnectionBase):
         self.assert_request_parameters({
             'Action': 'DescribeInstances',
             'MaxResults': 10},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp', 'Version'])
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp', 'Version'])
 
     def test_next_token(self):
         self.set_http_response(status_code=200)
@@ -1366,13 +1384,14 @@ class TestDescribeInstances(TestEC2ConnectionBase):
         self.assert_request_parameters({
             'Action': 'DescribeInstances',
             'NextToken': 'abcdefgh'},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp', 'Version'])
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp', 'Version'])
+
 
 class TestDescribeTags(TestEC2ConnectionBase):
 
     def default_body(self):
-        return """
+        return b"""
             <DescribeTagsResponse>
             </DescribeTagsResponse>
         """
@@ -1382,8 +1401,8 @@ class TestDescribeTags(TestEC2ConnectionBase):
         self.ec2.get_all_tags()
         self.assert_request_parameters({
             'Action': 'DescribeTags'},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp', 'Version'])
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp', 'Version'])
 
     def test_max_results(self):
         self.set_http_response(status_code=200)
@@ -1393,8 +1412,8 @@ class TestDescribeTags(TestEC2ConnectionBase):
         self.assert_request_parameters({
             'Action': 'DescribeTags',
             'MaxResults': 10},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp', 'Version'])
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp', 'Version'])
 
 
 class TestSignatureAlteration(TestEC2ConnectionBase):
@@ -1424,7 +1443,7 @@ class TestSignatureAlteration(TestEC2ConnectionBase):
 
 class TestAssociateAddress(TestEC2ConnectionBase):
     def default_body(self):
-        return """
+        return b"""
             <AssociateAddressResponse xmlns="http://ec2.amazonaws.com/doc/2013-10-15/">
                <requestId>59dbff89-35bd-4eac-99ed-be587EXAMPLE</requestId>
                <return>true</return>
@@ -1447,7 +1466,7 @@ class TestAssociateAddress(TestEC2ConnectionBase):
 
 class TestAssociateAddressFail(TestEC2ConnectionBase):
     def default_body(self):
-        return """
+        return b"""
             <Response>
                 <Errors>
                      <Error>
@@ -1469,9 +1488,9 @@ class TestAssociateAddressFail(TestEC2ConnectionBase):
 
 class TestDescribeVolumes(TestEC2ConnectionBase):
     def default_body(self):
-        return """
+        return b"""
             <DescribeVolumesResponse xmlns="http://ec2.amazonaws.com/doc/2014-02-01/">
-               <requestId>59dbff89-35bd-4eac-99ed-be587EXAMPLE</requestId> 
+               <requestId>59dbff89-35bd-4eac-99ed-be587EXAMPLE</requestId>
                <volumeSet>
                   <item>
                      <volumeId>vol-1a2b3c4d</volumeId>
@@ -1524,9 +1543,9 @@ class TestDescribeVolumes(TestEC2ConnectionBase):
             'Action': 'DescribeVolumes',
             'VolumeId.1': 'vol-1a2b3c4d',
             'VolumeId.2': 'vol-5e6f7a8b'},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp',
-                                   'Version'])
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp',
+                                  'Version'])
         self.assertEqual(len(result), 2)
         self.assertEqual(result[0].id, 'vol-1a2b3c4d')
         self.assertTrue(result[0].encrypted)
@@ -1536,9 +1555,9 @@ class TestDescribeVolumes(TestEC2ConnectionBase):
 
 class TestDescribeSnapshots(TestEC2ConnectionBase):
     def default_body(self):
-        return """
+        return b"""
             <DescribeSnapshotsResponse xmlns="http://ec2.amazonaws.com/doc/2014-02-01/">
-               <requestId>59dbff89-35bd-4eac-99ed-be587EXAMPLE</requestId> 
+               <requestId>59dbff89-35bd-4eac-99ed-be587EXAMPLE</requestId>
                <snapshotSet>
                   <item>
                      <snapshotId>snap-1a2b3c4d</snapshotId>
@@ -1577,9 +1596,9 @@ class TestDescribeSnapshots(TestEC2ConnectionBase):
             'Action': 'DescribeSnapshots',
             'SnapshotId.1': 'snap-1a2b3c4d',
             'SnapshotId.2': 'snap-5e6f7a8b'},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp',
-                                   'Version'])
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp',
+                                  'Version'])
         self.assertEqual(len(result), 2)
         self.assertEqual(result[0].id, 'snap-1a2b3c4d')
         self.assertTrue(result[0].encrypted)
@@ -1589,9 +1608,9 @@ class TestDescribeSnapshots(TestEC2ConnectionBase):
 
 class TestCreateVolume(TestEC2ConnectionBase):
     def default_body(self):
-        return """
+        return b"""
             <CreateVolumeResponse xmlns="http://ec2.amazonaws.com/doc/2014-05-01/">
-              <requestId>59dbff89-35bd-4eac-99ed-be587EXAMPLE</requestId> 
+              <requestId>59dbff89-35bd-4eac-99ed-be587EXAMPLE</requestId>
               <volumeId>vol-1a2b3c4d</volumeId>
               <size>80</size>
               <snapshotId/>
@@ -1613,9 +1632,9 @@ class TestCreateVolume(TestEC2ConnectionBase):
             'Size': 80,
             'SnapshotId': 'snap-1a2b3c4d',
             'Encrypted': 'true'},
-             ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
-                                   'SignatureVersion', 'Timestamp',
-                                   'Version'])
+            ignore_params_values=['AWSAccessKeyId', 'SignatureMethod',
+                                  'SignatureVersion', 'Timestamp',
+                                  'Version'])
         self.assertEqual(result.id, 'vol-1a2b3c4d')
         self.assertTrue(result.encrypted)
 
