@@ -1,8 +1,7 @@
 #!/usr/bin/env python
 from datetime import datetime, timedelta
 
-from tests.unit import unittest
-import mock
+from tests.compat import mock, unittest
 import os
 
 from boto import provider
@@ -113,6 +112,10 @@ class TestProvider(unittest.TestCase):
             }, 'profile prod': {
                 'aws_access_key_id': 'prod_access_key',
                 'aws_secret_access_key': 'prod_secret_key',
+            }, 'profile prod_withtoken': {
+                'aws_access_key_id': 'prod_access_key',
+                'aws_secret_access_key': 'prod_secret_key',
+                'aws_security_token': 'prod_token',
             }, 'Credentials': {
                 'aws_access_key_id': 'default_access_key',
                 'aws_secret_access_key': 'default_secret_key'
@@ -121,6 +124,10 @@ class TestProvider(unittest.TestCase):
         p = provider.Provider('aws', profile_name='prod')
         self.assertEqual(p.access_key, 'prod_access_key')
         self.assertEqual(p.secret_key, 'prod_secret_key')
+        p = provider.Provider('aws', profile_name='prod_withtoken')
+        self.assertEqual(p.access_key, 'prod_access_key')
+        self.assertEqual(p.secret_key, 'prod_secret_key')
+        self.assertEqual(p.security_token, 'prod_token')
         q = provider.Provider('aws', profile_name='dev')
         self.assertEqual(q.access_key, 'dev_access_key')
         self.assertEqual(q.secret_key, 'dev_secret_key')
@@ -402,7 +409,7 @@ class TestProvider(unittest.TestCase):
         self.assertEqual(p.access_key, 'cfg_access_key')
         self.assertEqual(p.secret_key, 'cfg_secret_key')
 
-    @mock.patch('os.path.exists', return_value=True)
+    @mock.patch('os.path.isfile', return_value=True)
     @mock.patch.object(provider.Config, 'load_from_path')
     def test_shared_config_loading(self, load_from_path, exists):
         provider.Provider('aws')
