@@ -20,19 +20,21 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 # IN THE SOFTWARE.
 #
-import mock
+from tests.compat import mock
 import re
 import xml.dom.minidom
-
 from boto.exception import BotoServerError
 from boto.route53.connection import Route53Connection
 from boto.route53.exception import DNSServerError
+from boto.route53.healthcheck import HealthCheck
 from boto.route53.record import ResourceRecordSets, Record
 from boto.route53.zone import Zone
 
 from nose.plugins.attrib import attr
-from tests.unit import unittest
 from tests.unit import AWSMockServiceTestCase
+from boto.compat import six
+urllib = six.moves.urllib
+
 
 @attr(route53=True)
 class TestRoute53Connection(AWSMockServiceTestCase):
@@ -45,14 +47,14 @@ class TestRoute53Connection(AWSMockServiceTestCase):
         }
 
     def default_body(self):
-        return """<Route53Result>
+        return b"""<Route53Result>
     <Message>It failed.</Message>
 </Route53Result>
 """
 
     def test_typical_400(self):
         self.set_http_response(status_code=400, header=[
-            ['Code', 'Throttling'],
+            ['Code', 'AccessDenied'],
         ])
 
         with self.assertRaises(DNSServerError) as err:
@@ -60,11 +62,22 @@ class TestRoute53Connection(AWSMockServiceTestCase):
 
         self.assertTrue('It failed.' in str(err.exception))
 
-    @mock.patch('time.sleep')
-    def test_retryable_400(self, sleep_mock):
+    def test_retryable_400_prior_request_not_complete(self):
+        # Test ability to retry on ``PriorRequestNotComplete``.
         self.set_http_response(status_code=400, header=[
             ['Code', 'PriorRequestNotComplete'],
         ])
+        self.do_retry_handler()
+
+    def test_retryable_400_throttling(self):
+        # Test ability to rety on ``Throttling``.
+        self.set_http_response(status_code=400, header=[
+            ['Code', 'Throttling'],
+        ])
+        self.do_retry_handler()
+
+    @mock.patch('time.sleep')
+    def do_retry_handler(self, sleep_mock):
 
         def incr_retry_handler(func):
             def _wrapper(*args, **kwargs):
@@ -88,6 +101,7 @@ class TestRoute53Connection(AWSMockServiceTestCase):
         # Unpatch.
         self.service_connection._retry_handler = orig_retry
 
+
 @attr(route53=True)
 class TestCreateZoneRoute53(AWSMockServiceTestCase):
     connection_class = Route53Connection
@@ -96,7 +110,7 @@ class TestCreateZoneRoute53(AWSMockServiceTestCase):
         super(TestCreateZoneRoute53, self).setUp()
 
     def default_body(self):
-        return """
+        return b"""
 <CreateHostedZoneResponse xmlns="https://route53.amazonaws.com/doc/2012-02-29/">
     <HostedZone>
         <Id>/hostedzone/Z11111</Id>
@@ -138,6 +152,7 @@ class TestCreateZoneRoute53(AWSMockServiceTestCase):
         self.assertEqual(response['CreateHostedZoneResponse']['DelegationSet']['NameServers'],
                          ['ns-100.awsdns-01.com', 'ns-1000.awsdns-01.co.uk', 'ns-1000.awsdns-01.org', 'ns-900.awsdns-01.net'])
 
+
 @attr(route53=True)
 class TestGetZoneRoute53(AWSMockServiceTestCase):
     connection_class = Route53Connection
@@ -146,7 +161,7 @@ class TestGetZoneRoute53(AWSMockServiceTestCase):
         super(TestGetZoneRoute53, self).setUp()
 
     def default_body(self):
-        return """
+        return b"""
 <ListHostedZonesResponse xmlns="https://route53.amazonaws.com/doc/2012-02-29/">
     <HostedZones>
         <HostedZone>
@@ -181,9 +196,9 @@ class TestGetZoneRoute53(AWSMockServiceTestCase):
         response = self.service_connection.get_all_hosted_zones()
 
         domains = ['example2.com.', 'example1.com.', 'example.com.']
-        print response['ListHostedZonesResponse']['HostedZones'][0]
+        print(response['ListHostedZonesResponse']['HostedZones'][0])
         for d in response['ListHostedZonesResponse']['HostedZones']:
-            print "Removing: %s" % d['Name']
+            print("Removing: %s" % d['Name'])
             domains.remove(d['Name'])
 
         self.assertEqual(domains, [])
@@ -195,6 +210,7 @@ class TestGetZoneRoute53(AWSMockServiceTestCase):
         self.assertTrue(isinstance(response, Zone))
         self.assertEqual(response.name, "example.com.")
 
+
 @attr(route53=True)
 class TestGetHostedZoneRoute53(AWSMockServiceTestCase):
     connection_class = Route53Connection
@@ -203,7 +219,7 @@ class TestGetHostedZoneRoute53(AWSMockServiceTestCase):
         super(TestGetHostedZoneRoute53, self).setUp()
 
     def default_body(self):
-        return """
+        return b"""
 <GetHostedZoneResponse xmlns="https://route53.amazonaws.com/doc/2012-02-29/">
     <HostedZone>
         <Id>/hostedzone/Z1111</Id>
@@ -232,6 +248,7 @@ class TestGetHostedZoneRoute53(AWSMockServiceTestCase):
         self.assertEqual(response['GetHostedZoneResponse']['DelegationSet']['NameServers'],
                          ['ns-1000.awsdns-40.org', 'ns-200.awsdns-30.com', 'ns-900.awsdns-50.net', 'ns-1000.awsdns-00.co.uk'])
 
+
 @attr(route53=True)
 class TestGetAllRRSetsRoute53(AWSMockServiceTestCase):
     connection_class = Route53Connection
@@ -240,7 +257,7 @@ class TestGetAllRRSetsRoute53(AWSMockServiceTestCase):
         super(TestGetAllRRSetsRoute53, self).setUp()
 
     def default_body(self):
-        return """
+        return b"""
 <ListResourceRecordSetsResponse xmlns="https://route53.amazonaws.com/doc/2013-04-01/">
     <ResourceRecordSets>
         <ResourceRecordSet>
@@ -273,6 +290,7 @@ class TestGetAllRRSetsRoute53(AWSMockServiceTestCase):
                 <EvaluateTargetHealth>true</EvaluateTargetHealth>
                 <DNSName>example-123456-evaluate-health.us-west-2.elb.amazonaws.com.</DNSName>
             </AliasTarget>
+            <HealthCheckId>abcdefgh-abcd-abcd-abcd-abcdefghijkl</HealthCheckId>
         </ResourceRecordSet>
         <ResourceRecordSet>
             <Name>us-west-2-no-evaluate-health.example.com.</Name>
@@ -284,6 +302,7 @@ class TestGetAllRRSetsRoute53(AWSMockServiceTestCase):
                 <EvaluateTargetHealth>false</EvaluateTargetHealth>
                 <DNSName>example-123456-no-evaluate-health.us-west-2.elb.amazonaws.com.</DNSName>
             </AliasTarget>
+            <HealthCheckId>abcdefgh-abcd-abcd-abcd-abcdefghijkl</HealthCheckId>
         </ResourceRecordSet>
         <ResourceRecordSet>
             <Name>failover.example.com.</Name>
@@ -297,6 +316,18 @@ class TestGetAllRRSetsRoute53(AWSMockServiceTestCase):
                 </ResourceRecord>
             </ResourceRecords>
         </ResourceRecordSet>
+        <ResourceRecordSet>
+            <Name>us-west-2-evaluate-health-healthcheck.example.com.</Name>
+            <Type>A</Type>
+            <SetIdentifier>latency-example-us-west-2-evaluate-health-healthcheck</SetIdentifier>
+            <Region>us-west-2</Region>
+            <AliasTarget>
+                <HostedZoneId>ABCDEFG123456</HostedZoneId>
+                <EvaluateTargetHealth>true</EvaluateTargetHealth>
+                <DNSName>example-123456-evaluate-health-healthcheck.us-west-2.elb.amazonaws.com.</DNSName>
+            </AliasTarget>
+            <HealthCheckId>076a32f8-86f7-4c9e-9fa2-c163d5be67d9</HealthCheckId>
+        </ResourceRecordSet>
     </ResourceRecordSets>
     <IsTruncated>false</IsTruncated>
     <MaxItems>100</MaxItems>
@@ -307,8 +338,9 @@ class TestGetAllRRSetsRoute53(AWSMockServiceTestCase):
         self.set_http_response(status_code=200)
         response = self.service_connection.get_all_rrsets("Z1111", "A", "example.com.")
 
-        self.assertEqual(self.actual_request.path,
-                         "/2013-04-01/hostedzone/Z1111/rrset?type=A&name=example.com.")
+        self.assertIn(self.actual_request.path,
+                      ("/2013-04-01/hostedzone/Z1111/rrset?type=A&name=example.com.",
+                       "/2013-04-01/hostedzone/Z1111/rrset?name=example.com.&type=A"))
 
         self.assertTrue(isinstance(response, ResourceRecordSets))
         self.assertEqual(response.hosted_zone_id, "Z1111")
@@ -327,6 +359,7 @@ class TestGetAllRRSetsRoute53(AWSMockServiceTestCase):
         self.assertTrue(evaluate_record.alias_evaluate_target_health)
         self.assertEqual(evaluate_record.alias_dns_name, 'example-123456-evaluate-health.us-west-2.elb.amazonaws.com.')
         evaluate_xml = evaluate_record.to_xml()
+        self.assertTrue(evaluate_record.health_check, 'abcdefgh-abcd-abcd-abcd-abcdefghijkl')
         self.assertTrue('<EvaluateTargetHealth>true</EvaluateTargetHealth>' in evaluate_xml)
 
         no_evaluate_record = response[3]
@@ -338,8 +371,9 @@ class TestGetAllRRSetsRoute53(AWSMockServiceTestCase):
         self.assertFalse(no_evaluate_record.alias_evaluate_target_health)
         self.assertEqual(no_evaluate_record.alias_dns_name, 'example-123456-no-evaluate-health.us-west-2.elb.amazonaws.com.')
         no_evaluate_xml = no_evaluate_record.to_xml()
+        self.assertTrue(no_evaluate_record.health_check, 'abcdefgh-abcd-abcd-abcd-abcdefghijkl')
         self.assertTrue('<EvaluateTargetHealth>false</EvaluateTargetHealth>' in no_evaluate_xml)
-        
+
         failover_record = response[4]
         self.assertEqual(failover_record.name, 'failover.example.com.')
         self.assertEqual(failover_record.type, 'A')
@@ -347,6 +381,196 @@ class TestGetAllRRSetsRoute53(AWSMockServiceTestCase):
         self.assertEqual(failover_record.failover, 'PRIMARY')
         self.assertEqual(failover_record.ttl, '60')
 
+        healthcheck_record = response[5]
+        self.assertEqual(healthcheck_record.health_check, '076a32f8-86f7-4c9e-9fa2-c163d5be67d9')
+        self.assertEqual(healthcheck_record.name, 'us-west-2-evaluate-health-healthcheck.example.com.')
+        self.assertEqual(healthcheck_record.identifier, 'latency-example-us-west-2-evaluate-health-healthcheck')
+        self.assertEqual(healthcheck_record.alias_dns_name, 'example-123456-evaluate-health-healthcheck.us-west-2.elb.amazonaws.com.')
+
+
+@attr(route53=True)
+class TestTruncatedGetAllRRSetsRoute53(AWSMockServiceTestCase):
+    connection_class = Route53Connection
+
+    def setUp(self):
+        super(TestTruncatedGetAllRRSetsRoute53, self).setUp()
+
+    def default_body(self):
+        return b"""
+<ListResourceRecordSetsResponse xmlns="https://route53.amazonaws.com/doc/2013-04-01/">
+  <ResourceRecordSets>
+    <ResourceRecordSet>
+      <Name>example.com.</Name>
+      <Type>NS</Type>
+      <TTL>900</TTL>
+      <ResourceRecords>
+        <ResourceRecord>
+          <Value>ns-91.awsdns-41.co.uk.</Value>
+        </ResourceRecord>
+        <ResourceRecord>
+          <Value>ns-1929.awsdns-93.net.</Value>
+        </ResourceRecord>
+        <ResourceRecord>
+          <Value>ns-12.awsdns-21.org.</Value>
+        </ResourceRecord>
+        <ResourceRecord>
+          <Value>ns-102.awsdns-96.com.</Value>
+        </ResourceRecord>
+      </ResourceRecords>
+    </ResourceRecordSet>
+    <ResourceRecordSet>
+      <Name>example.com.</Name>
+      <Type>SOA</Type>
+      <TTL>1800</TTL>
+      <ResourceRecords>
+        <ResourceRecord>
+          <Value>ns-1929.awsdns-93.net. hostmaster.awsdns.net. 1 10800 3600 604800 1800</Value>
+        </ResourceRecord>
+      </ResourceRecords>
+    </ResourceRecordSet>
+    <ResourceRecordSet>
+      <Name>wrr.example.com.</Name>
+      <Type>A</Type>
+      <SetIdentifier>primary</SetIdentifier>
+      <Weight>100</Weight>
+      <TTL>300</TTL>
+      <ResourceRecords>
+        <ResourceRecord><Value>127.0.0.1</Value></ResourceRecord>
+      </ResourceRecords>
+    </ResourceRecordSet>
+  </ResourceRecordSets>
+  <IsTruncated>true</IsTruncated>
+  <NextRecordName>wrr.example.com.</NextRecordName>
+  <NextRecordType>A</NextRecordType>
+  <NextRecordIdentifier>secondary</NextRecordIdentifier>
+  <MaxItems>3</MaxItems>
+</ListResourceRecordSetsResponse>"""
+
+    def paged_body(self):
+        return b"""
+<ListResourceRecordSetsResponse xmlns="https://route53.amazonaws.com/doc/2013-04-01/">
+  <ResourceRecordSets>
+    <ResourceRecordSet>
+      <Name>wrr.example.com.</Name>
+      <Type>A</Type>
+      <SetIdentifier>secondary</SetIdentifier>
+      <Weight>50</Weight>
+      <TTL>300</TTL>
+      <ResourceRecords>
+        <ResourceRecord><Value>127.0.0.2</Value></ResourceRecord>
+      </ResourceRecords>
+    </ResourceRecordSet>
+  </ResourceRecordSets>
+  <IsTruncated>false</IsTruncated>
+  <MaxItems>3</MaxItems>
+</ListResourceRecordSetsResponse>"""
+
+
+    def test_get_all_rr_sets(self):
+        self.set_http_response(status_code=200)
+        response = self.service_connection.get_all_rrsets("Z1111", maxitems=3)
+
+        # made first request
+        self.assertEqual(self.actual_request.path, '/2013-04-01/hostedzone/Z1111/rrset?maxitems=3')
+
+        # anticipate a second request when we page it
+        self.set_http_response(status_code=200, body=self.paged_body())
+
+        # this should trigger another call to get_all_rrsets
+        self.assertEqual(len(list(response)), 4)
+
+        url_parts = urllib.parse.urlparse(self.actual_request.path)
+        self.assertEqual(url_parts.path, '/2013-04-01/hostedzone/Z1111/rrset')
+        self.assertEqual(urllib.parse.parse_qs(url_parts.query),
+                         dict(type=['A'], name=['wrr.example.com.'], identifier=['secondary']))
+
+
+@attr(route53=True)
+class TestCreateHealthCheckRoute53IpAddress(AWSMockServiceTestCase):
+    connection_class = Route53Connection
+
+    def setUp(self):
+        super(TestCreateHealthCheckRoute53IpAddress, self).setUp()
+
+    def default_body(self):
+        return b"""
+<CreateHealthCheckResponse xmlns="https://route53.amazonaws.com/doc/2013-04-01/">
+   <HealthCheck>
+      <Id>34778cf8-e31e-4974-bad0-b108bd1623d3</Id>
+      <CallerReference>2fa48c8f-76ef-4253-9874-8bcb2b0d7694</CallerReference>
+      <HealthCheckConfig>
+         <IPAddress>74.125.228.81</IPAddress>
+         <Port>443</Port>
+         <Type>HTTPS_STR_MATCH</Type>
+         <SearchString>OK</SearchString>
+         <ResourcePath>/health_check</ResourcePath>
+         <RequestInterval>30</RequestInterval>
+         <FailureThreshold>3</FailureThreshold>
+      </HealthCheckConfig>
+   </HealthCheck>
+</CreateHealthCheckResponse>
+        """
+
+    def test_create_health_check_ip_address(self):
+        self.set_http_response(status_code=201)
+        hc = HealthCheck(ip_addr='74.125.228.81', port=443, hc_type='HTTPS_STR_MATCH', resource_path='/health_check', string_match='OK')
+        hc_xml = hc.to_xml()
+        self.assertFalse('<FullyQualifiedDomainName>' in hc_xml)
+        self.assertTrue('<IPAddress>' in hc_xml)
+
+        response = self.service_connection.create_health_check(hc)
+        hc_resp = response['CreateHealthCheckResponse']['HealthCheck']['HealthCheckConfig']
+        self.assertEqual(hc_resp['IPAddress'], '74.125.228.81')
+        self.assertEqual(hc_resp['ResourcePath'], '/health_check')
+        self.assertEqual(hc_resp['Type'], 'HTTPS_STR_MATCH')
+        self.assertEqual(hc_resp['Port'], '443')
+        self.assertEqual(hc_resp['ResourcePath'], '/health_check')
+        self.assertEqual(hc_resp['SearchString'], 'OK')
+        self.assertEqual(response['CreateHealthCheckResponse']['HealthCheck']['Id'], '34778cf8-e31e-4974-bad0-b108bd1623d3')
+
+
+@attr(route53=True)
+class TestCreateHealthCheckRoute53FQDN(AWSMockServiceTestCase):
+    connection_class = Route53Connection
+
+    def setUp(self):
+        super(TestCreateHealthCheckRoute53FQDN, self).setUp()
+
+    def default_body(self):
+        return b"""
+<CreateHealthCheckResponse xmlns="https://route53.amazonaws.com/doc/2013-04-01/">
+   <HealthCheck>
+      <Id>f9abfe10-8d2a-4bbd-8f35-796f0f8572f2</Id>
+      <CallerReference>3246ac17-b651-4295-a5c8-c132a59693d7</CallerReference>
+      <HealthCheckConfig>
+         <Port>443</Port>
+         <Type>HTTPS</Type>
+         <ResourcePath>/health_check</ResourcePath>
+         <FullyQualifiedDomainName>example.com</FullyQualifiedDomainName>
+         <RequestInterval>30</RequestInterval>
+         <FailureThreshold>3</FailureThreshold>
+      </HealthCheckConfig>
+   </HealthCheck>
+</CreateHealthCheckResponse>
+        """
+
+    def test_create_health_check_fqdn(self):
+        self.set_http_response(status_code=201)
+        hc = HealthCheck(ip_addr='', port=443, hc_type='HTTPS', resource_path='/health_check', fqdn='example.com')
+        hc_xml = hc.to_xml()
+        self.assertTrue('<FullyQualifiedDomainName>' in hc_xml)
+        self.assertFalse('<IPAddress>' in hc_xml)
+
+        response = self.service_connection.create_health_check(hc)
+        hc_resp = response['CreateHealthCheckResponse']['HealthCheck']['HealthCheckConfig']
+        self.assertEqual(hc_resp['FullyQualifiedDomainName'], 'example.com')
+        self.assertEqual(hc_resp['ResourcePath'], '/health_check')
+        self.assertEqual(hc_resp['Type'], 'HTTPS')
+        self.assertEqual(hc_resp['Port'], '443')
+        self.assertEqual(hc_resp['ResourcePath'], '/health_check')
+        self.assertEqual(response['CreateHealthCheckResponse']['HealthCheck']['Id'], 'f9abfe10-8d2a-4bbd-8f35-796f0f8572f2')
+
+
 @attr(route53=True)
 class TestChangeResourceRecordSetsRoute53(AWSMockServiceTestCase):
     connection_class = Route53Connection
@@ -355,7 +579,7 @@ class TestChangeResourceRecordSetsRoute53(AWSMockServiceTestCase):
         super(TestChangeResourceRecordSetsRoute53, self).setUp()
 
     def default_body(self):
-        return """
+        return b"""
 <ChangeResourceRecordSetsResponse xmlns="https://route53.amazonaws.com/doc/2013-04-01/">
     <ChangeInfo>
         <Id>/change/C1111111111111</Id>
@@ -372,13 +596,13 @@ class TestChangeResourceRecordSetsRoute53(AWSMockServiceTestCase):
         rrsets.add_change_record('CREATE', Record('wrr.example.com', 'CNAME', 60, ['cname.target'], weight=10, identifier='weight-1'))
         rrsets.add_change_record('CREATE', Record('lbr.example.com', 'TXT', 60, ['text record'], region='us-west-2', identifier='region-1'))
         rrsets.add_change_record('CREATE', Record('failover.example.com', 'A', 60, ['2.2.2.2'], health_check='hc-1234', failover='PRIMARY', identifier='primary'))
-        
+
         changes_xml = rrsets.to_xml()
-        
+
         # the whitespacing doesn't match exactly, so we'll pretty print and drop all new lines
-        # not the best, but 
+        # not the best, but
         actual_xml = re.sub(r"\s*[\r\n]+", "\n", xml.dom.minidom.parseString(changes_xml).toprettyxml())
-        expected_xml = re.sub(r"\s*[\r\n]+", "\n", xml.dom.minidom.parseString("""
+        expected_xml = re.sub(r"\s*[\r\n]+", "\n", xml.dom.minidom.parseString(b"""
 <ChangeResourceRecordSetsRequest xmlns="https://route53.amazonaws.com/doc/2013-04-01/">
     <ChangeBatch>
         <Comment>None</Comment>
@@ -458,7 +682,6 @@ class TestChangeResourceRecordSetsRoute53(AWSMockServiceTestCase):
     </ChangeBatch>
 </ChangeResourceRecordSetsRequest>
         """).toprettyxml())
-        
+
         # Note: the alias XML should not include the TTL, even if it's specified in the object model
         self.assertEqual(actual_xml, expected_xml)
-
